searchData={"items":[{"type":"module","title":":zigler","doc":"Parse transform module for using Zigler with erlang.\n\nFor the canonical example, see:\nhttps://www.erlang.org/doc/man/erl_id_trans.html","ref":":zigler.html"},{"type":"module","title":"Prerequisites - :zigler","doc":"In order to use Zigler in an erlang project, you must have the Elixir\nruntime.   You may do this any way you wish, but Zigler recommends\nrebar_mix:\n\nhttps://github.com/Supersonido/rebar_mix\n\nThere are instructions on how to make sure Elixir is available at\ncompile time for your erlang project.","ref":":zigler.html#module-prerequisites"},{"type":"module","title":"Building a Zig Module - :zigler","doc":"General documentation on parse transforms is very light.  To use zigler as\na parse transform:\n\n```erlang\n-module(my_erlang_module).\n-compile({parse_transform, zigler}).\n-export([...]).\n\n-zig_code(\"\npub fn hello_world() [] const u8 {\n  return \"Hello, world!\";\n}\n\")\n\n-zig_opts([{otp_app, my_app}]).\n```\n\nThis creates the `hello_world/0` function in your\nmodule which returns the \"Hello, world!\" binary.\n\nfor options to be delivered in the `zig_opts` attribute, see the\n`Zig` module documentation.\n\nNote that the `...` for the `nifs` option is not representable in erlang AST.\nInstead, use the atom `auto`.\n\n> #","ref":":zigler.html#module-building-a-zig-module"},{"type":"module","title":"Note {: .warning } - :zigler","doc":">\n> Erlang integration is highly experimental and the interface\n> may be changed in the future.","ref":":zigler.html#module-note-warning"},{"type":"function","title":":zigler.parse_transform/2","doc":"performs a parse transformation on the AST for an erlang module,\nconverting public functions in the","ref":":zigler.html#parse_transform/2"},{"type":"module","title":"Zig","doc":"Inline NIF support for [Zig](https://ziglang.org)\n\n#","ref":"Zig.html"},{"type":"module","title":"Motivation - Zig","doc":"> Zig is a general-purpose programming language designed for robustness,\n> optimality, and maintainability.\n\nThe programming philosophy of Zig matches up nicely with the programming\nphilosophy of the BEAM VM and in particular its emphasis on simplicity and\nstructure should very appealing to the practitioners of Elixir.\n\nThe following features make Zig extremely amenable to inline language\nsupport in a BEAM language:\n\n- simplicity.  Zig's syntax is definable in a simple YACC document and\n  Zig takes a stance against making its featureset more complex (though\n  it may evolve somewhat en route to 1.0)\n- Composability.  Zig is unopinionated about how to go about memory\n  allocations.  Its allocator interface is very easily able to be backed\n  by the BEAM's, which means that you have access to generic memory\n  allocation *strategies* through its composable allocator scheme.\n- C integration.  It's very easy to design C-interop between Zig and C.\n  Zigler has been designed to make it easier to use Zigler to build\n  C libraries than to use C directly see [Easy C](#module-easy-c).\n\n#","ref":"Zig.html#module-motivation"},{"type":"module","title":"Guides - Zig","doc":"Please consult the following guides for detailed topics:\n\n- [Using Nifs](1-nifs.html)\n- [Collection datatypes](2-collections.html)\n- [Allocator strategies](3-allocators.html)\n- [Nif options](4-nif_options.html)\n- [Resources](5-nif_options.html)\n- [C integration](6-c_integration.html)\n- [Concurrency strategies](7-concurrency.html)\n- [Global module options](8-module_options.html)\n\n> #","ref":"Zig.html#module-guides"},{"type":"module","title":"Zig version support {: .warning } - Zig","doc":">\n> although the large-scale archictecture of zigler is settled,\n> zigler features may break backwards compatibility until zig reaches\n> 1.0\n\n#","ref":"Zig.html#module-zig-version-support-warning"},{"type":"module","title":"Nerves Support - Zig","doc":"Nerves is supported out of the box, and Zigler will be able to seamlessly\ndetect the cross-compilation information (os, architecture, runtime) and\nbuild correctly for that target.\n\n#","ref":"Zig.html#module-nerves-support"},{"type":"module","title":"Basic usage - Zig","doc":"In the BEAM, you can define a NIF by consulting the following [document](\nhttps://erlang.org/doc/man/erl_nif.html) and implementing the appropriate\nshared object/DLL callbacks.  However, Zigler will take care of all of\nthis for you.\n\nSimply `use Zig` in your module, providing the app atom in the property\nlist.\n\nThen, use the `sigil_Z/2` macro and write zig code.  To present a function\nas a nif in your module, simply export it from your code namespace by\nmaking it a `pub` function in your zig code.\n\n##","ref":"Zig.html#module-basic-usage"},{"type":"module","title":"Example - Zig","doc":"```\ndefmodule MyModule do\n  use Zig, otp_app: :my_app\n\n  ~Z\"\"\"\n  pub fn my_func(val: i64) i64 {\n      return val + 1;\n  }\n  \"\"\"\n\nend\n```\n\nZig will *automatically* fill out the appropriate NIF C template, compile\nthe shared object, and bind it into the module pre-compilation. In the case\nof the example, there will be a `MyModule.my_func/1` function call found in\nthe module.\n\nZig will also make sure that your statically-typed Zig data are guarded\nwhen you marshal it from the dynamically-typed BEAM world.  However, you may\nonly pass in and return certain types.  As an escape hatch, you may use\nthe [`beam.term`](beam.html#term) type which is a wrapped\n[`ERL_NIF_TERM`](https://www.erlang.org/doc/man/erl_nif.html#ERL_NIF_TERM) type.\nSee [`erl_nif`](https://www.erlang.org/doc/man/erl_nif.html).\n\n##","ref":"Zig.html#module-example"},{"type":"module","title":"Environment - Zig","doc":"For many functions, you'll need to import the [`beam`](beam.html) package and\ncreate a function that takes a [`beam.env`](beam.html#env) as its first\nargument.  This will enable you to directly access or create wrapped beam\nterm data.  The equivalent of the above code will be:\n\n##","ref":"Zig.html#module-environment"},{"type":"module","title":"Example - Zig","doc":"```\ndefmodule MyModule do\n  use Zig, otp_app: :my_app\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n\n  pub fn my_func(env: beam.env, val_term: beam.term) !beam.term {\n      const val = try beam.get(i64, env, val_term, .{});\n      return beam.make(env, val + 1, .{});\n  }\n  \"\"\"\nend\n```\n\nFor more details on [`get`](beam.html#get) and [`make`](beam.html#make)\nfunctions see the [`beam`](beam.html) documentation.\n\n> ##","ref":"Zig.html#module-example"},{"type":"module","title":"Manual Term marshalling {: .warning } - Zig","doc":">\n> If you don't use automatic marshalling, Zigler will not be able\n> to provide the following conveniences:\n>\n> - argument error details.  The zig code will raise a generic\n>   BEAM `ArgumentError` but it won't have specific details about\n>   what the expected type was and which argument was in error.\n>\n> - dialyzer type information for your function.  You will have\n>   to supply that type information in your nif configuration.\n\n#","ref":"Zig.html#module-manual-term-marshalling-warning"},{"type":"module","title":"Functions missing from [`beam`](beam.html) - Zig","doc":"The `beam` module doesn't comprehensively contain all nif functions.\nFor functions that correspond to [`erl_nif.h`](https://www.erlang.org/doc/man/erl_nif.html)\nyou can import the erl_nif package, which has the `erl_nif` C API\n\n```zig\nconst erl_nif = @import(\"erl_nif\");\n```\n\n#","ref":"Zig.html#module-functions-missing-from-beam-beam-html"},{"type":"module","title":"Importing external files - Zig","doc":"If you need to write zig code outside of the module, just place it in\nthe same directory as your module.\n\n##","ref":"Zig.html#module-importing-external-files"},{"type":"module","title":"Example - Zig","doc":"```elixir\n~Z\"\"\"\nconst extra_code = @import(\"extra_code.zig\");\n\npub fn use_extra_code(val: i64) i64 {\n  return extra_code.extra_fn(val);\n}\n\npub const forwarded_function = extra_code.forwarded_function;\n\"\"\"\n```\n\nIf you would like to include a custom c header file, create an subdirectory\nof your module's directory and add it as an available include directory,\nas shown here (in this case the subdirectory is called `include`).  The\nZig build system will add the include path(s) in the analysis and\ncompilation pipelines.\n\n```elixir\ndefmodule MyModule\n  use Zig,\n    otp_app: :my_app,\n    include_dir: \"include\"\n\n  ~Z\"\"\"\n  const c = @cImport({\n    @cInclude(\"my_c_header.h\");\n  });\n  ...\n  \"\"\"\nend\n```\n\nIf the c header defines `extern` functions, it's your responsibility to make\nsure those externed functions are available by\n[compiling other c files](#module-compiling-c-c-files) or\n[using an external library](#module-external-libraries).\n\n#","ref":"Zig.html#module-example"},{"type":"module","title":"External Libraries - Zig","doc":"If you need to bind static (`*.a`) or dynamic (`*.so`) libraries into your\nmodule, you may link them with the `:libs` argument.\n\nNote that zig statically binds shared libraries into the assets it creates.\nThis simplifies deployment for you.\n\n##","ref":"Zig.html#module-external-libraries"},{"type":"module","title":"Example (explicit library path) - Zig","doc":"```elixir\ndefmodule Blas do\n  use Zig,\n    otp_app: :my_app,\n    link_lib: \"path/to/libblas.a\"\n\n  ~Z\"\"\"\n  const blas = @cImport({\n    @cInclude(\"cblas.h\");\n  ...\n  \"\"\"\nend\n```\n\nYou can also link system libraries.  This relies on `zig build`'s ability\nto locate system libraries.  Note that you will need to follow your system's\nlibrary convention, for example in the case of linux, that means removing the\n\"lib\" prefix and the \".so\" extension.\n\n##","ref":"Zig.html#module-example-explicit-library-path"},{"type":"module","title":"Example (system libraries) - Zig","doc":"```elixir\ndefmodule Blas do\n  use Zig,\n    otp_app: :my_app,\n    link_lib: {:system, \"blas\"}\n\n  ~Z\"\"\"\n  const blas = @cImport({\n    @cInclude(\"cblas.h\");\n  ...\n  \"\"\"\nend\n```\n\n#","ref":"Zig.html#module-example-system-libraries"},{"type":"module","title":"Compiling C/C++ files - Zig","doc":"You can direct zigler to compile C or C++ files that are in\nyour directory tree.  Currently, you must explicitly pick each file, in the\nfuture, there may be support for directories (and selecting compile options)\nbased on customizeable rules.\n\nTo do this, fill the \"sources\" option with a list of files (represented as\nstrings), or a file/options pair (represented as a tuple).\n\n```elixir\ndefmodule UsesCOrCpp do\n  use Zig,\n    otp_app: :my_app,\n    link_libcpp: true,  # note: optional for c-only code\n    include_dir: [\"include\"],\n    c_src: [\n      \"some_c_source.c\",\n      {\"some_cpp_source.cpp\", [\"-std=c++17\"]},\n      {\"directory_of_files/*\", [\"-std=c99\"]},\n    ]\n\n  ~Z\"\"\"\n  ...\n  \"\"\"\nend\n```\n\n#","ref":"Zig.html#module-compiling-c-c-files"},{"type":"module","title":"Easy C - Zig","doc":"In some cases, you may have a C project that ships with a library and a\nheader file that you would like to mount as NIF functions in your module.\nIn this case, you can use the `easy_c` option to automate the work of\nstitching your library into the module.  Note that in this case, you must\ndeclare all of the function that you would like to import.  Here is an\nexample of importing three functions from the blas example as above.\n\nFor details of what the nif options mean, see: `Zig.EasyC`\n\n```elixir\ndefmodule BlasWithEasyC do\n  use Zig,\n    otp_app: :my_app,\n    easy_c: \"cblas.h\",\n    link_lib: {:system, \"blas\"},\n    nifs: [\n      :cblas_dasum,\n      cblas_daxpy: [return: [4, length: {:arg, 0}]],\n      daxpy_bin: [alias: :cblas_daxpy, return: [4, :binary, length: {:arg, 0}]]\n    ]\nend\n```\n\n#","ref":"Zig.html#module-easy-c"},{"type":"module","title":"Compilation debug - Zig","doc":"If something should go wrong, Zigler will translate the Zig compiler error\ninto an Elixir compiler error, and let you know which line in the\n`~Z` block it came from.\n\n#","ref":"Zig.html#module-compilation-debug"},{"type":"module","title":"Documentation - Zig","doc":"Use the builtin zig `///` docstring to write your documentation.  If it's in\nfront of the nif declaration, it will wind up in the correct place in your\nelixir documentation.\n\nNote that the `//!` docstring is not supported.  Use `@moduledoc` instead.\n\n#","ref":"Zig.html#module-documentation"},{"type":"module","title":"Bring your own version of Zig - Zig","doc":"If you would like to use your system's local `zig` command, specify\nthis in your `use Zig` statement options.\n\n```elixir\nuse Zig, otp_app: :my_app, local_zig: true\n```\n\nThis will use `System.find_executable/1` to obtain the zig command. If\nyou want to specify a specific zig path, use the following:\n\n```elixir\nuse Zig, otp_app: :my_app, zig_path: \"path/to/zig/command\"\n```","ref":"Zig.html#module-bring-your-own-version-of-zig"},{"type":"function","title":"Zig.code/1","doc":"retrieves the zig code from any given module that was compiled with zigler","ref":"Zig.html#code/1"},{"type":"function","title":"Zig.nif_name/2","doc":"outputs a String name for the module.\n\nnote that for filesystem use, you must supply the extension.  For internal (BEAM) use, the\nfilesystem extension will be inferred.  Therefore we provide two versions of this function.","ref":"Zig.html#nif_name/2"},{"type":"macro","title":"Zig.sigil_Z/2","doc":"declares a string block to be included in the module's .zig source file.","ref":"Zig.html#sigil_Z/2"},{"type":"macro","title":"Zig.sigil_z/2","doc":"like `sigil_Z/2`, but lets you interpolate values from the outside\nelixir context using string interpolation (the `#{value}` form)","ref":"Zig.html#sigil_z/2"},{"type":"function","title":"Zig.version/0","doc":"default version of zig supported by this version of zigler.\n\n> #","ref":"Zig.html#version/0"},{"type":"function","title":"API warning {: .warning } - Zig.version/0","doc":">\n> this API may change in the future.","ref":"Zig.html#version/0-api-warning-warning"},{"type":"protocol","title":"Zig.Type","doc":"","ref":"Zig.Type.html"},{"type":"function","title":"Zig.Type.error_prongs/2","doc":"catch prongs to correctly perform error handling, atom is a reference to function in `Zig.ErrorProng`","ref":"Zig.Type.html#error_prongs/2"},{"type":"function","title":"Zig.Type.from_json/2","doc":"","ref":"Zig.Type.html#from_json/2"},{"type":"function","title":"Zig.Type.get_result/2","doc":"generates make clauses in zig","ref":"Zig.Type.html#get_result/2"},{"type":"function","title":"Zig.Type.marshal_param/4","doc":"","ref":"Zig.Type.html#marshal_param/4"},{"type":"function","title":"Zig.Type.marshal_return/3","doc":"","ref":"Zig.Type.html#marshal_return/3"},{"type":"function","title":"Zig.Type.marshals_param?/1","doc":"beam-side type conversions that might be necessary to get an elixir parameter into a zig parameter","ref":"Zig.Type.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.marshals_return?/1","doc":"beam-side type conversions that might be necessary to get a zig return into an elixir return","ref":"Zig.Type.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.missing_size?/1","doc":"","ref":"Zig.Type.html#missing_size?/1"},{"type":"function","title":"Zig.Type.needs_make?/1","doc":"","ref":"Zig.Type.html#needs_make?/1"},{"type":"function","title":"Zig.Type.parse/1","doc":"","ref":"Zig.Type.html#parse/1"},{"type":"function","title":"Zig.Type.return_allowed?/1","doc":"","ref":"Zig.Type.html#return_allowed?/1"},{"type":"macro","title":"Zig.Type.sigil_t/2","doc":"","ref":"Zig.Type.html#sigil_t/2"},{"type":"function","title":"Zig.Type.spec/1","doc":"","ref":"Zig.Type.html#spec/1"},{"type":"function","title":"Zig.Type.spec/3","doc":"","ref":"Zig.Type.html#spec/3"},{"type":"function","title":"Zig.Type.to_call/1","doc":"","ref":"Zig.Type.html#to_call/1"},{"type":"type","title":"Zig.Type.t/0","doc":"","ref":"Zig.Type.html#t:t/0"},{"type":"module","title":"Zig.Type.Array","doc":"","ref":"Zig.Type.Array.html"},{"type":"function","title":"Zig.Type.Array.error_prongs/2","doc":"","ref":"Zig.Type.Array.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Array.from_json/2","doc":"","ref":"Zig.Type.Array.html#from_json/2"},{"type":"function","title":"Zig.Type.Array.get_result/2","doc":"","ref":"Zig.Type.Array.html#get_result/2"},{"type":"function","title":"Zig.Type.Array.marshal_param/4","doc":"","ref":"Zig.Type.Array.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Array.marshal_return/3","doc":"","ref":"Zig.Type.Array.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Array.marshals_param?/1","doc":"","ref":"Zig.Type.Array.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Array.marshals_return?/1","doc":"","ref":"Zig.Type.Array.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Array.missing_size?/1","doc":"","ref":"Zig.Type.Array.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Array.needs_make?/1","doc":"","ref":"Zig.Type.Array.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Array.of/3","doc":"","ref":"Zig.Type.Array.html#of/3"},{"type":"function","title":"Zig.Type.Array.return_allowed?/1","doc":"","ref":"Zig.Type.Array.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Array.spec/3","doc":"","ref":"Zig.Type.Array.html#spec/3"},{"type":"function","title":"Zig.Type.Array.to_call/1","doc":"","ref":"Zig.Type.Array.html#to_call/1"},{"type":"function","title":"Zig.Type.Array.to_string/1","doc":"","ref":"Zig.Type.Array.html#to_string/1"},{"type":"type","title":"Zig.Type.Array.t/0","doc":"","ref":"Zig.Type.Array.html#t:t/0"},{"type":"module","title":"Zig.Type.Bool","doc":"","ref":"Zig.Type.Bool.html"},{"type":"function","title":"Zig.Type.Bool.error_prongs/2","doc":"","ref":"Zig.Type.Bool.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Bool.from_json/1","doc":"","ref":"Zig.Type.Bool.html#from_json/1"},{"type":"function","title":"Zig.Type.Bool.get_result/2","doc":"","ref":"Zig.Type.Bool.html#get_result/2"},{"type":"function","title":"Zig.Type.Bool.marshal_param/4","doc":"","ref":"Zig.Type.Bool.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Bool.marshal_return/3","doc":"","ref":"Zig.Type.Bool.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Bool.marshals_param?/1","doc":"","ref":"Zig.Type.Bool.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Bool.marshals_return?/1","doc":"","ref":"Zig.Type.Bool.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Bool.missing_size?/1","doc":"","ref":"Zig.Type.Bool.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Bool.needs_make?/1","doc":"","ref":"Zig.Type.Bool.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Bool.return_allowed?/1","doc":"","ref":"Zig.Type.Bool.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Bool.spec/3","doc":"","ref":"Zig.Type.Bool.html#spec/3"},{"type":"function","title":"Zig.Type.Bool.to_call/1","doc":"","ref":"Zig.Type.Bool.html#to_call/1"},{"type":"function","title":"Zig.Type.Bool.to_string/1","doc":"","ref":"Zig.Type.Bool.html#to_string/1"},{"type":"type","title":"Zig.Type.Bool.t/0","doc":"","ref":"Zig.Type.Bool.html#t:t/0"},{"type":"module","title":"Zig.Type.Cpointer","doc":"","ref":"Zig.Type.Cpointer.html"},{"type":"function","title":"Zig.Type.Cpointer.error_prongs/2","doc":"","ref":"Zig.Type.Cpointer.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Cpointer.from_json/2","doc":"","ref":"Zig.Type.Cpointer.html#from_json/2"},{"type":"function","title":"Zig.Type.Cpointer.get_result/2","doc":"","ref":"Zig.Type.Cpointer.html#get_result/2"},{"type":"function","title":"Zig.Type.Cpointer.marshal_param/4","doc":"","ref":"Zig.Type.Cpointer.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Cpointer.marshal_return/3","doc":"","ref":"Zig.Type.Cpointer.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Cpointer.marshals_param?/1","doc":"","ref":"Zig.Type.Cpointer.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Cpointer.marshals_return?/1","doc":"","ref":"Zig.Type.Cpointer.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Cpointer.missing_size?/1","doc":"","ref":"Zig.Type.Cpointer.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Cpointer.needs_make?/1","doc":"","ref":"Zig.Type.Cpointer.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Cpointer.of/1","doc":"","ref":"Zig.Type.Cpointer.html#of/1"},{"type":"function","title":"Zig.Type.Cpointer.return_allowed?/1","doc":"","ref":"Zig.Type.Cpointer.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Cpointer.spec/3","doc":"","ref":"Zig.Type.Cpointer.html#spec/3"},{"type":"function","title":"Zig.Type.Cpointer.to_call/1","doc":"","ref":"Zig.Type.Cpointer.html#to_call/1"},{"type":"function","title":"Zig.Type.Cpointer.to_string/1","doc":"","ref":"Zig.Type.Cpointer.html#to_string/1"},{"type":"type","title":"Zig.Type.Cpointer.t/0","doc":"","ref":"Zig.Type.Cpointer.html#t:t/0"},{"type":"module","title":"Zig.Type.Enum","doc":"","ref":"Zig.Type.Enum.html"},{"type":"function","title":"Zig.Type.Enum.error_prongs/2","doc":"","ref":"Zig.Type.Enum.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Enum.from_json/2","doc":"","ref":"Zig.Type.Enum.html#from_json/2"},{"type":"function","title":"Zig.Type.Enum.get_result/2","doc":"","ref":"Zig.Type.Enum.html#get_result/2"},{"type":"function","title":"Zig.Type.Enum.inspect/2","doc":"","ref":"Zig.Type.Enum.html#inspect/2"},{"type":"function","title":"Zig.Type.Enum.marshal_param/4","doc":"","ref":"Zig.Type.Enum.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Enum.marshal_return/3","doc":"","ref":"Zig.Type.Enum.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Enum.marshals_param?/1","doc":"","ref":"Zig.Type.Enum.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Enum.marshals_return?/1","doc":"","ref":"Zig.Type.Enum.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Enum.missing_size?/1","doc":"","ref":"Zig.Type.Enum.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Enum.needs_make?/1","doc":"","ref":"Zig.Type.Enum.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Enum.return_allowed?/1","doc":"","ref":"Zig.Type.Enum.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Enum.spec/3","doc":"","ref":"Zig.Type.Enum.html#spec/3"},{"type":"function","title":"Zig.Type.Enum.to_call/1","doc":"","ref":"Zig.Type.Enum.html#to_call/1"},{"type":"function","title":"Zig.Type.Enum.to_string/1","doc":"","ref":"Zig.Type.Enum.html#to_string/1"},{"type":"type","title":"Zig.Type.Enum.t/0","doc":"","ref":"Zig.Type.Enum.html#t:t/0"},{"type":"module","title":"Zig.Type.Error","doc":"","ref":"Zig.Type.Error.html"},{"type":"function","title":"Zig.Type.Error.error_prongs/2","doc":"","ref":"Zig.Type.Error.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Error.from_json/2","doc":"","ref":"Zig.Type.Error.html#from_json/2"},{"type":"function","title":"Zig.Type.Error.get_result/2","doc":"","ref":"Zig.Type.Error.html#get_result/2"},{"type":"function","title":"Zig.Type.Error.marshal_param/4","doc":"","ref":"Zig.Type.Error.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Error.marshal_return/3","doc":"","ref":"Zig.Type.Error.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Error.marshals_param?/1","doc":"","ref":"Zig.Type.Error.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Error.marshals_return?/1","doc":"","ref":"Zig.Type.Error.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Error.missing_size?/1","doc":"","ref":"Zig.Type.Error.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Error.needs_make?/1","doc":"","ref":"Zig.Type.Error.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Error.of/1","doc":"","ref":"Zig.Type.Error.html#of/1"},{"type":"function","title":"Zig.Type.Error.return_allowed?/1","doc":"","ref":"Zig.Type.Error.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Error.spec/3","doc":"","ref":"Zig.Type.Error.html#spec/3"},{"type":"function","title":"Zig.Type.Error.to_call/1","doc":"","ref":"Zig.Type.Error.html#to_call/1"},{"type":"function","title":"Zig.Type.Error.to_string/1","doc":"","ref":"Zig.Type.Error.html#to_string/1"},{"type":"type","title":"Zig.Type.Error.t/0","doc":"","ref":"Zig.Type.Error.html#t:t/0"},{"type":"module","title":"Zig.Type.Float","doc":"","ref":"Zig.Type.Float.html"},{"type":"function","title":"Zig.Type.Float.error_prongs/2","doc":"","ref":"Zig.Type.Float.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Float.from_json/1","doc":"","ref":"Zig.Type.Float.html#from_json/1"},{"type":"function","title":"Zig.Type.Float.get_result/2","doc":"","ref":"Zig.Type.Float.html#get_result/2"},{"type":"function","title":"Zig.Type.Float.inspect/2","doc":"","ref":"Zig.Type.Float.html#inspect/2"},{"type":"function","title":"Zig.Type.Float.marshal_param/4","doc":"","ref":"Zig.Type.Float.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Float.marshal_return/3","doc":"","ref":"Zig.Type.Float.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Float.marshals_param?/1","doc":"","ref":"Zig.Type.Float.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Float.marshals_return?/1","doc":"","ref":"Zig.Type.Float.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Float.missing_size?/1","doc":"","ref":"Zig.Type.Float.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Float.needs_make?/1","doc":"","ref":"Zig.Type.Float.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Float.parse/1","doc":"","ref":"Zig.Type.Float.html#parse/1"},{"type":"function","title":"Zig.Type.Float.return_allowed?/1","doc":"","ref":"Zig.Type.Float.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Float.spec/3","doc":"","ref":"Zig.Type.Float.html#spec/3"},{"type":"function","title":"Zig.Type.Float.to_call/1","doc":"","ref":"Zig.Type.Float.html#to_call/1"},{"type":"function","title":"Zig.Type.Float.to_string/1","doc":"","ref":"Zig.Type.Float.html#to_string/1"},{"type":"type","title":"Zig.Type.Float.t/0","doc":"","ref":"Zig.Type.Float.html#t:t/0"},{"type":"module","title":"Zig.Type.Function","doc":"module representing the zig type, as identified by performing semantic\nanalysis on the zig code.","ref":"Zig.Type.Function.html"},{"type":"function","title":"Zig.Type.Function.from_json/2","doc":"","ref":"Zig.Type.Function.html#from_json/2"},{"type":"type","title":"Zig.Type.Function.t/0","doc":"","ref":"Zig.Type.Function.html#t:t/0"},{"type":"module","title":"Zig.Type.Integer","doc":"","ref":"Zig.Type.Integer.html"},{"type":"function","title":"Zig.Type.Integer.error_prongs/2","doc":"","ref":"Zig.Type.Integer.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Integer.from_json/1","doc":"","ref":"Zig.Type.Integer.html#from_json/1"},{"type":"function","title":"Zig.Type.Integer.get_bits!/2","doc":"","ref":"Zig.Type.Integer.html#get_bits!/2"},{"type":"function","title":"Zig.Type.Integer.get_result/2","doc":"","ref":"Zig.Type.Integer.html#get_result/2"},{"type":"function","title":"Zig.Type.Integer.inspect/2","doc":"","ref":"Zig.Type.Integer.html#inspect/2"},{"type":"function","title":"Zig.Type.Integer.marshal_param/4","doc":"","ref":"Zig.Type.Integer.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Integer.marshal_return/3","doc":"","ref":"Zig.Type.Integer.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Integer.marshals_param?/1","doc":"","ref":"Zig.Type.Integer.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Integer.marshals_return?/1","doc":"","ref":"Zig.Type.Integer.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Integer.missing_size?/1","doc":"","ref":"Zig.Type.Integer.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Integer.needs_make?/1","doc":"","ref":"Zig.Type.Integer.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Integer.parse/1","doc":"","ref":"Zig.Type.Integer.html#parse/1"},{"type":"function","title":"Zig.Type.Integer.return_allowed?/1","doc":"","ref":"Zig.Type.Integer.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Integer.spec/3","doc":"","ref":"Zig.Type.Integer.html#spec/3"},{"type":"function","title":"Zig.Type.Integer.to_call/1","doc":"","ref":"Zig.Type.Integer.html#to_call/1"},{"type":"function","title":"Zig.Type.Integer.to_string/1","doc":"","ref":"Zig.Type.Integer.html#to_string/1"},{"type":"type","title":"Zig.Type.Integer.t/0","doc":"","ref":"Zig.Type.Integer.html#t:t/0"},{"type":"module","title":"Zig.Type.Manypointer","doc":"","ref":"Zig.Type.Manypointer.html"},{"type":"function","title":"Zig.Type.Manypointer.error_prongs/2","doc":"","ref":"Zig.Type.Manypointer.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Manypointer.from_json/2","doc":"","ref":"Zig.Type.Manypointer.html#from_json/2"},{"type":"function","title":"Zig.Type.Manypointer.get_result/2","doc":"","ref":"Zig.Type.Manypointer.html#get_result/2"},{"type":"function","title":"Zig.Type.Manypointer.marshal_param/4","doc":"","ref":"Zig.Type.Manypointer.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Manypointer.marshal_return/3","doc":"","ref":"Zig.Type.Manypointer.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Manypointer.marshals_param?/1","doc":"","ref":"Zig.Type.Manypointer.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Manypointer.marshals_return?/1","doc":"","ref":"Zig.Type.Manypointer.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Manypointer.missing_size?/1","doc":"","ref":"Zig.Type.Manypointer.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Manypointer.needs_make?/1","doc":"","ref":"Zig.Type.Manypointer.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Manypointer.of/2","doc":"","ref":"Zig.Type.Manypointer.html#of/2"},{"type":"function","title":"Zig.Type.Manypointer.return_allowed?/1","doc":"","ref":"Zig.Type.Manypointer.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Manypointer.spec/3","doc":"","ref":"Zig.Type.Manypointer.html#spec/3"},{"type":"function","title":"Zig.Type.Manypointer.to_call/1","doc":"","ref":"Zig.Type.Manypointer.html#to_call/1"},{"type":"function","title":"Zig.Type.Manypointer.to_string/1","doc":"","ref":"Zig.Type.Manypointer.html#to_string/1"},{"type":"type","title":"Zig.Type.Manypointer.t/0","doc":"","ref":"Zig.Type.Manypointer.html#t:t/0"},{"type":"module","title":"Zig.Type.Optional","doc":"","ref":"Zig.Type.Optional.html"},{"type":"function","title":"Zig.Type.Optional.error_prongs/2","doc":"","ref":"Zig.Type.Optional.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Optional.from_json/2","doc":"","ref":"Zig.Type.Optional.html#from_json/2"},{"type":"function","title":"Zig.Type.Optional.get_result/2","doc":"","ref":"Zig.Type.Optional.html#get_result/2"},{"type":"function","title":"Zig.Type.Optional.marshal_param/4","doc":"","ref":"Zig.Type.Optional.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Optional.marshal_return/3","doc":"","ref":"Zig.Type.Optional.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Optional.marshals_param?/1","doc":"","ref":"Zig.Type.Optional.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Optional.marshals_return?/1","doc":"","ref":"Zig.Type.Optional.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Optional.missing_size?/1","doc":"","ref":"Zig.Type.Optional.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Optional.needs_make?/1","doc":"","ref":"Zig.Type.Optional.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Optional.of/1","doc":"","ref":"Zig.Type.Optional.html#of/1"},{"type":"function","title":"Zig.Type.Optional.return_allowed?/1","doc":"","ref":"Zig.Type.Optional.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Optional.spec/3","doc":"","ref":"Zig.Type.Optional.html#spec/3"},{"type":"function","title":"Zig.Type.Optional.to_call/1","doc":"","ref":"Zig.Type.Optional.html#to_call/1"},{"type":"function","title":"Zig.Type.Optional.to_string/1","doc":"","ref":"Zig.Type.Optional.html#to_string/1"},{"type":"type","title":"Zig.Type.Optional.t/0","doc":"","ref":"Zig.Type.Optional.html#t:t/0"},{"type":"module","title":"Zig.Type.Resource","doc":"","ref":"Zig.Type.Resource.html"},{"type":"function","title":"Zig.Type.Resource.error_prongs/2","doc":"","ref":"Zig.Type.Resource.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Resource.from_json/2","doc":"","ref":"Zig.Type.Resource.html#from_json/2"},{"type":"function","title":"Zig.Type.Resource.get_result/2","doc":"","ref":"Zig.Type.Resource.html#get_result/2"},{"type":"function","title":"Zig.Type.Resource.marshal_param/4","doc":"","ref":"Zig.Type.Resource.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Resource.marshal_return/3","doc":"","ref":"Zig.Type.Resource.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Resource.marshals_param?/1","doc":"","ref":"Zig.Type.Resource.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Resource.marshals_return?/1","doc":"","ref":"Zig.Type.Resource.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Resource.missing_size?/1","doc":"","ref":"Zig.Type.Resource.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Resource.needs_make?/1","doc":"","ref":"Zig.Type.Resource.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Resource.return_allowed?/1","doc":"","ref":"Zig.Type.Resource.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Resource.spec/3","doc":"","ref":"Zig.Type.Resource.html#spec/3"},{"type":"function","title":"Zig.Type.Resource.to_call/1","doc":"","ref":"Zig.Type.Resource.html#to_call/1"},{"type":"function","title":"Zig.Type.Resource.to_string/1","doc":"","ref":"Zig.Type.Resource.html#to_string/1"},{"type":"type","title":"Zig.Type.Resource.t/0","doc":"","ref":"Zig.Type.Resource.html#t:t/0"},{"type":"module","title":"Zig.Type.Slice","doc":"","ref":"Zig.Type.Slice.html"},{"type":"function","title":"Zig.Type.Slice.error_prongs/2","doc":"","ref":"Zig.Type.Slice.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Slice.from_json/2","doc":"","ref":"Zig.Type.Slice.html#from_json/2"},{"type":"function","title":"Zig.Type.Slice.get_result/2","doc":"","ref":"Zig.Type.Slice.html#get_result/2"},{"type":"function","title":"Zig.Type.Slice.marshal_param/4","doc":"","ref":"Zig.Type.Slice.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Slice.marshal_return/3","doc":"","ref":"Zig.Type.Slice.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Slice.marshals_param?/1","doc":"","ref":"Zig.Type.Slice.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Slice.marshals_return?/1","doc":"","ref":"Zig.Type.Slice.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Slice.missing_size?/1","doc":"","ref":"Zig.Type.Slice.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Slice.needs_make?/1","doc":"","ref":"Zig.Type.Slice.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Slice.of/2","doc":"","ref":"Zig.Type.Slice.html#of/2"},{"type":"function","title":"Zig.Type.Slice.return_allowed?/1","doc":"","ref":"Zig.Type.Slice.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Slice.spec/3","doc":"","ref":"Zig.Type.Slice.html#spec/3"},{"type":"function","title":"Zig.Type.Slice.to_call/1","doc":"","ref":"Zig.Type.Slice.html#to_call/1"},{"type":"function","title":"Zig.Type.Slice.to_string/1","doc":"","ref":"Zig.Type.Slice.html#to_string/1"},{"type":"type","title":"Zig.Type.Slice.t/0","doc":"","ref":"Zig.Type.Slice.html#t:t/0"},{"type":"module","title":"Zig.Type.Struct","doc":"","ref":"Zig.Type.Struct.html"},{"type":"function","title":"Zig.Type.Struct.error_prongs/2","doc":"","ref":"Zig.Type.Struct.html#error_prongs/2"},{"type":"function","title":"Zig.Type.Struct.from_json/2","doc":"","ref":"Zig.Type.Struct.html#from_json/2"},{"type":"function","title":"Zig.Type.Struct.get_result/2","doc":"","ref":"Zig.Type.Struct.html#get_result/2"},{"type":"function","title":"Zig.Type.Struct.marshal_param/2","doc":"","ref":"Zig.Type.Struct.html#marshal_param/2"},{"type":"function","title":"Zig.Type.Struct.marshal_param/4","doc":"","ref":"Zig.Type.Struct.html#marshal_param/4"},{"type":"function","title":"Zig.Type.Struct.marshal_return/3","doc":"","ref":"Zig.Type.Struct.html#marshal_return/3"},{"type":"function","title":"Zig.Type.Struct.marshals_param?/1","doc":"","ref":"Zig.Type.Struct.html#marshals_param?/1"},{"type":"function","title":"Zig.Type.Struct.marshals_return?/1","doc":"","ref":"Zig.Type.Struct.html#marshals_return?/1"},{"type":"function","title":"Zig.Type.Struct.missing_size?/1","doc":"","ref":"Zig.Type.Struct.html#missing_size?/1"},{"type":"function","title":"Zig.Type.Struct.needs_make?/1","doc":"","ref":"Zig.Type.Struct.html#needs_make?/1"},{"type":"function","title":"Zig.Type.Struct.return_allowed?/1","doc":"","ref":"Zig.Type.Struct.html#return_allowed?/1"},{"type":"function","title":"Zig.Type.Struct.spec/3","doc":"","ref":"Zig.Type.Struct.html#spec/3"},{"type":"function","title":"Zig.Type.Struct.to_call/1","doc":"","ref":"Zig.Type.Struct.html#to_call/1"},{"type":"function","title":"Zig.Type.Struct.to_string/1","doc":"","ref":"Zig.Type.Struct.html#to_string/1"},{"type":"type","title":"Zig.Type.Struct.t/0","doc":"","ref":"Zig.Type.Struct.html#t:t/0"},{"type":"module","title":"Zig.Assembler","doc":"Assembles the staging directory for zig content.\n\nThe staging directory contains:\n\n- `build.zig` for the library file.","ref":"Zig.Assembler.html"},{"type":"function","title":"Zig.Assembler.assemble/2","doc":"","ref":"Zig.Assembler.html#assemble/2"},{"type":"function","title":"Zig.Assembler.directory/1","doc":"staging directory for the zigler assembly","ref":"Zig.Assembler.html#directory/1"},{"type":"module","title":"Zig.Builder","doc":"Code for interfacing with `std.build.Builder`, the interface for programmatically invoking\nbuild code with the `zig build` command.","ref":"Zig.Builder.html"},{"type":"function","title":"Zig.Builder.build/2","doc":"","ref":"Zig.Builder.html#build/2"},{"type":"module","title":"Zig.Command","doc":"contains all parts of the Zig library involved in calling the\nzig compiler toolchain, especially with regards to the `zig` command, except\nfor assembling the build.zig file, which is performed by the\n`Zig.Builder` module.","ref":"Zig.Command.html"},{"type":"function","title":"Zig.Command.compile/2","doc":"","ref":"Zig.Command.html#compile/2"},{"type":"function","title":"Zig.Command.executable_path/1","doc":"","ref":"Zig.Command.html#executable_path/1"},{"type":"function","title":"Zig.Command.fmt/1","doc":"","ref":"Zig.Command.html#fmt/1"},{"type":"function","title":"Zig.Command.get_arch/0","doc":"","ref":"Zig.Command.html#get_arch/0"},{"type":"function","title":"Zig.Command.get_os/0","doc":"","ref":"Zig.Command.html#get_os/0"},{"type":"function","title":"Zig.Command.run_sema/2","doc":"","ref":"Zig.Command.html#run_sema/2"},{"type":"function","title":"Zig.Command.targets/0","doc":"","ref":"Zig.Command.html#targets/0"},{"type":"module","title":"Zig.Compiler","doc":"handles instrumenting elixir code with hooks for zig NIFs.","ref":"Zig.Compiler.html"},{"type":"function","title":"Zig.Compiler.assembly_dir/2","doc":"","ref":"Zig.Compiler.html#assembly_dir/2"},{"type":"function","title":"Zig.Compiler.compile/4","doc":"","ref":"Zig.Compiler.html#compile/4"},{"type":"module","title":"Zig.Sema","doc":"","ref":"Zig.Sema.html"},{"type":"function","title":"Zig.Sema.analyze_file!/2","doc":"","ref":"Zig.Sema.html#analyze_file!/2"},{"type":"function","title":"Zig.Sema.run_sema/3","doc":"","ref":"Zig.Sema.html#run_sema/3"},{"type":"function","title":"Zig.Sema.run_sema!/3","doc":"","ref":"Zig.Sema.html#run_sema!/3"},{"type":"module","title":"Zig.EasyC","doc":"","ref":"Zig.EasyC.html"},{"type":"function","title":"Zig.EasyC.build_from/1","doc":"","ref":"Zig.EasyC.html#build_from/1"},{"type":"function","title":"Zig.EasyC.normalize_aliasing/1","doc":"","ref":"Zig.EasyC.html#normalize_aliasing/1"},{"type":"module","title":"Zig.Nif.Basic","doc":"Architecture:\n\nSynchronous has two different cases.  The first case is that the nif can be called\ndirectly.  In this case, the function is mapped directly to function name.  In the\ncase that the nif needs marshalling, the function is mapped to `marshalled- `.\nand the called function contains wrapping logic.\n\nTo understand wrapping logic, see `Zig.Nif.Marshaller`","ref":"Zig.Nif.Basic.html"},{"type":"function","title":"Zig.Nif.Basic.cleanup_for/3","doc":"","ref":"Zig.Nif.Basic.html#cleanup_for/3"},{"type":"function","title":"Zig.Nif.Basic.context/1","doc":"","ref":"Zig.Nif.Basic.html#context/1"},{"type":"function","title":"Zig.Nif.Basic.entrypoint/1","doc":"","ref":"Zig.Nif.Basic.html#entrypoint/1"},{"type":"function","title":"Zig.Nif.Basic.render_elixir/1","doc":"","ref":"Zig.Nif.Basic.html#render_elixir/1"},{"type":"function","title":"Zig.Nif.Basic.render_erlang/1","doc":"","ref":"Zig.Nif.Basic.html#render_erlang/1"},{"type":"function","title":"Zig.Nif.Basic.render_zig/1","doc":"","ref":"Zig.Nif.Basic.html#render_zig/1"},{"type":"function","title":"Zig.Nif.Basic.resources/1","doc":"","ref":"Zig.Nif.Basic.html#resources/1"},{"type":"function","title":"Zig.Nif.Basic.type_cleanup/2","doc":"","ref":"Zig.Nif.Basic.html#type_cleanup/2"},{"type":"behaviour","title":"Zig.Nif.Concurrency","doc":"behaviour module which describes the interface for \"plugins\" which\ngenerate concurrency-specific code.","ref":"Zig.Nif.Concurrency.html"},{"type":"callback","title":"Zig.Nif.Concurrency.render_elixir/1","doc":"","ref":"Zig.Nif.Concurrency.html#c:render_elixir/1"},{"type":"callback","title":"Zig.Nif.Concurrency.render_erlang/1","doc":"","ref":"Zig.Nif.Concurrency.html#c:render_erlang/1"},{"type":"callback","title":"Zig.Nif.Concurrency.render_zig/1","doc":"","ref":"Zig.Nif.Concurrency.html#c:render_zig/1"},{"type":"callback","title":"Zig.Nif.Concurrency.resources/1","doc":"","ref":"Zig.Nif.Concurrency.html#c:resources/1"},{"type":"callback","title":"Zig.Nif.Concurrency.table_entries/1","doc":"returns \"table_entry\" tuples which are then used to generate the nif table.\nif a nif function needs multiple parts, for example, for concurrency\nmanagement, then multiple entries should be returned.","ref":"Zig.Nif.Concurrency.html#c:table_entries/1"},{"type":"type","title":"Zig.Nif.Concurrency.concurrency/0","doc":"","ref":"Zig.Nif.Concurrency.html#t:concurrency/0"},{"type":"type","title":"Zig.Nif.Concurrency.table_entry/0","doc":"","ref":"Zig.Nif.Concurrency.html#t:table_entry/0"},{"type":"module","title":"Zig.Nif.DirtyCpu","doc":"Dirty Cpu Nifs run dirty, on the cpu","ref":"Zig.Nif.DirtyCpu.html"},{"type":"module","title":"Zig.Nif.DirtyIo","doc":"Dirty Cpu Nifs run dirty, on the cpu","ref":"Zig.Nif.DirtyIo.html"},{"type":"module","title":"Zig.Nif.Synchronous","doc":"","ref":"Zig.Nif.Synchronous.html"},{"type":"module","title":"Zig.Nif.Threaded","doc":"","ref":"Zig.Nif.Threaded.html"},{"type":"module","title":"Zig.Manifest","doc":"","ref":"Zig.Manifest.html"},{"type":"function","title":"Zig.Manifest.create/1","doc":"","ref":"Zig.Manifest.html#create/1"},{"type":"function","title":"Zig.Manifest.resolve/3","doc":"","ref":"Zig.Manifest.html#resolve/3"},{"type":"macro","title":"Zig.Manifest.resolver/2","doc":"","ref":"Zig.Manifest.html#resolver/2"},{"type":"type","title":"Zig.Manifest.t/0","doc":"","ref":"Zig.Manifest.html#t:t/0"},{"type":"module","title":"Zig.Module","doc":"abstraction representing multiple zig nif functions bound into a single\nmodule","ref":"Zig.Module.html"},{"type":"function","title":"Zig.Module.module_file/1","doc":"","ref":"Zig.Module.html#module_file/1"},{"type":"function","title":"Zig.Module.render_elixir/5","doc":"","ref":"Zig.Module.html#render_elixir/5"},{"type":"function","title":"Zig.Module.render_erlang/5","doc":"","ref":"Zig.Module.html#render_erlang/5"},{"type":"function","title":"Zig.Module.render_zig/4","doc":"","ref":"Zig.Module.html#render_zig/4"},{"type":"module","title":"Zig.Nif","doc":"module encapsulating all of the information required to correctly generate\na nif function.\n\nNote that all information obtained from semantic analysis of the function is\nstashed in the `Zig.Nif.Function` module.","ref":"Zig.Nif.html"},{"type":"function","title":"Zig.Nif.indexed_args/1","doc":"","ref":"Zig.Nif.html#indexed_args/1"},{"type":"function","title":"Zig.Nif.indexed_parameters/1","doc":"","ref":"Zig.Nif.html#indexed_parameters/1"},{"type":"function","title":"Zig.Nif.maybe_catch/1","doc":"","ref":"Zig.Nif.html#maybe_catch/1"},{"type":"function","title":"Zig.Nif.new/2","doc":"based on nif options for this function keyword at (opts :: nifs :: function_name)","ref":"Zig.Nif.html#new/2"},{"type":"function","title":"Zig.Nif.render_elixir/1","doc":"","ref":"Zig.Nif.html#render_elixir/1"},{"type":"function","title":"Zig.Nif.render_erlang/2","doc":"","ref":"Zig.Nif.html#render_erlang/2"},{"type":"function","title":"Zig.Nif.render_zig/1","doc":"","ref":"Zig.Nif.html#render_zig/1"},{"type":"function","title":"Zig.Nif.spec/1","doc":"","ref":"Zig.Nif.html#spec/1"},{"type":"function","title":"Zig.Nif.table_entries/1","doc":"","ref":"Zig.Nif.html#table_entries/1"},{"type":"function","title":"Zig.Nif.validate_return!/3","doc":"","ref":"Zig.Nif.html#validate_return!/3"},{"type":"type","title":"Zig.Nif.t/0","doc":"","ref":"Zig.Nif.html#t:t/0"},{"type":"module","title":"Zig.Resources","doc":"","ref":"Zig.Resources.html"},{"type":"function","title":"Zig.Resources.init_resource_type/2","doc":"","ref":"Zig.Resources.html#init_resource_type/2"},{"type":"function","title":"Zig.Resources.render/1","doc":"","ref":"Zig.Resources.html#render/1"},{"type":"function","title":"Zig.Resources.resource_decl/1","doc":"","ref":"Zig.Resources.html#resource_decl/1"},{"type":"function","title":"Zig.Resources.resource_prong/1","doc":"","ref":"Zig.Resources.html#resource_prong/1"},{"type":"type","title":"Zig.Resources.t/0","doc":"","ref":"Zig.Resources.html#t:t/0"},{"type":"module","title":"Zig.Analyzer","doc":"tools to analyze AST generated by Zig.Parser","ref":"Zig.Analyzer.html"},{"type":"function","title":"Zig.Analyzer.find/2","doc":"","ref":"Zig.Analyzer.html#find/2"},{"type":"function","title":"Zig.Analyzer.info_for/2","doc":"","ref":"Zig.Analyzer.html#info_for/2"},{"type":"function","title":"Zig.Analyzer.translate_location/3","doc":"","ref":"Zig.Analyzer.html#translate_location/3"},{"type":"module","title":"Zig.ErrorProng","doc":"","ref":"Zig.ErrorProng.html"},{"type":"function","title":"Zig.ErrorProng.argument_error_prong/2","doc":"","ref":"Zig.ErrorProng.html#argument_error_prong/2"},{"type":"function","title":"Zig.ErrorProng.error_return_prong/2","doc":"","ref":"Zig.ErrorProng.html#error_return_prong/2"},{"type":"module","title":"Zig.Macro","doc":"","ref":"Zig.Macro.html"},{"type":"function","title":"Zig.Macro.inspect/2","doc":"","ref":"Zig.Macro.html#inspect/2"},{"type":"module","title":"Zig.Options","doc":"parses and normalizes zig options.\n\nAlso sets up\n\n`options.zig` file which is mapped to `@import(\"zigler_options\")` in\n`beam.zig`.  This is then exposed as `@import(\"beam\").options` in your code.","ref":"Zig.Options.html"},{"type":"function","title":"Zig.Options.elixir_normalize!/1","doc":"Performs early normalization of options.  For Elixir only, this converts\nall AST representations which must be escaped before moving on to presenting\noptions to the Zigler compiler.","ref":"Zig.Options.html#elixir_normalize!/1"},{"type":"function","title":"Zig.Options.erlang_normalize!/1","doc":"","ref":"Zig.Options.html#erlang_normalize!/1"},{"type":"function","title":"Zig.Options.normalize!/1","doc":"","ref":"Zig.Options.html#normalize!/1"},{"type":"function","title":"Zig.Options.normalize_nifs/1","doc":"","ref":"Zig.Options.html#normalize_nifs/1"},{"type":"function","title":"Zig.Options.replace_lazy/3","doc":"","ref":"Zig.Options.html#replace_lazy/3"},{"type":"function","title":"Zig.Options.set_auto/2","doc":"","ref":"Zig.Options.html#set_auto/2"},{"type":"module","title":"Zig.QuoteErl","doc":"","ref":"Zig.QuoteErl.html"},{"type":"function","title":"Zig.QuoteErl.quote_erl/2","doc":"","ref":"Zig.QuoteErl.html#quote_erl/2"},{"type":"module","title":"Zig.Target","doc":"interfaces for cross-compilation logic.\n\nthis function primarily exists to support Nerves deployments, though\nit is possible to set an arbitrary cross-compilation target using a\nsetting in your `use Zig` directive.  This selects the architecture\nby checking your \"CC\" environment variable, which is in turn set by\nNerves, then adjusts gcc's machine type to a string which allows zig to\nselect the appropriate cross-compilation settings and libc.","ref":"Zig.Target.html"},{"type":"function","title":"Zig.Target.for_builder/1","doc":"","ref":"Zig.Target.html#for_builder/1"},{"type":"function","title":"Zig.Target.resolve/0","doc":"","ref":"Zig.Target.html#resolve/0"},{"type":"type","title":"Zig.Target.t/0","doc":"","ref":"Zig.Target.html#t:t/0"},{"type":"exception","title":"Zig.CompileError","doc":"","ref":"Zig.CompileError.html"},{"type":"function","title":"Zig.CompileError.message/1","doc":"","ref":"Zig.CompileError.html#message/1"},{"type":"function","title":"Zig.CompileError.to_error/2","doc":"","ref":"Zig.CompileError.html#to_error/2"},{"type":"exception","title":"Zig.Type.ParseError","doc":"","ref":"Zig.Type.ParseError.html"},{"type":"function","title":"Zig.Type.ParseError.message/1","doc":"","ref":"Zig.Type.ParseError.html#message/1"},{"type":"module","title":"beam","doc":"","ref":"beam.html"},{"type":null,"title":"beam.alloc_env/0","doc":"","ref":"beam.html#alloc_env"},{"type":null,"title":"beam.allocator/0","doc":"","ref":"beam.html#allocator"},{"type":null,"title":"beam.binary_to_slice/1","doc":"","ref":"beam.html#binary_to_slice"},{"type":null,"title":"beam.binary_to_term/2","doc":"","ref":"beam.html#binary_to_term"},{"type":null,"title":"beam.cleanup/2","doc":"","ref":"beam.html#cleanup"},{"type":null,"title":"beam.compare/2","doc":"","ref":"beam.html#compare"},{"type":null,"title":"beam.context/0","doc":"","ref":"beam.html#context"},{"type":null,"title":"beam.copy/2","doc":"","ref":"beam.html#copy"},{"type":null,"title":"beam.free_env/1","doc":"","ref":"beam.html#free_env"},{"type":null,"title":"beam.general_purpose_allocator/0","doc":"","ref":"beam.html#general_purpose_allocator"},{"type":null,"title":"beam.get/4","doc":"","ref":"beam.html#get"},{"type":null,"title":"beam.large_allocator/0","doc":"","ref":"beam.html#large_allocator"},{"type":null,"title":"beam.make/3","doc":"","ref":"beam.html#make"},{"type":null,"title":"beam.make_empty_list/1","doc":"","ref":"beam.html#make_empty_list"},{"type":null,"title":"beam.make_error_atom/1","doc":"","ref":"beam.html#make_error_atom"},{"type":null,"title":"beam.make_error_pair/3","doc":"","ref":"beam.html#make_error_pair"},{"type":null,"title":"beam.make_general_purpose_allocator_instance/0","doc":"","ref":"beam.html#make_general_purpose_allocator_instance"},{"type":null,"title":"beam.make_into_atom/2","doc":"","ref":"beam.html#make_into_atom"},{"type":null,"title":"beam.make_list_cell/3","doc":"","ref":"beam.html#make_list_cell"},{"type":null,"title":"beam.make_pid/2","doc":"","ref":"beam.html#make_pid"},{"type":null,"title":"beam.make_ref/1","doc":"","ref":"beam.html#make_ref"},{"type":null,"title":"beam.make_stacktrace/2","doc":"","ref":"beam.html#make_stacktrace"},{"type":null,"title":"beam.raise_elixir_exception/3","doc":"","ref":"beam.html#raise_elixir_exception"},{"type":null,"title":"beam.raise_exception/2","doc":"","ref":"beam.html#raise_exception"},{"type":null,"title":"beam.raise_with_error_return/3","doc":"","ref":"beam.html#raise_with_error_return"},{"type":null,"title":"beam.raw_allocator/0","doc":"","ref":"beam.html#raw_allocator"},{"type":null,"title":"beam.release_binary/1","doc":"","ref":"beam.html#release_binary"},{"type":null,"title":"beam.self/1","doc":"","ref":"beam.html#self"},{"type":null,"title":"beam.send/3","doc":"","ref":"beam.html#send"},{"type":null,"title":"beam.term_to_binary/2","doc":"","ref":"beam.html#term_to_binary"},{"type":null,"title":"beam.yield/1","doc":"","ref":"beam.html#yield"},{"type":"type","title":"beam.Compared/0","doc":"","ref":"beam.html#Compared"},{"type":"type","title":"beam.ExecutionContext/0","doc":"","ref":"beam.html#ExecutionContext"},{"type":"type","title":"beam.Payload/1","doc":"","ref":"beam.html#Payload"},{"type":"type","title":"beam.Resource/3","doc":"","ref":"beam.html#Resource"},{"type":"type","title":"beam.TermType/0","doc":"","ref":"beam.html#TermType"},{"type":"type","title":"beam.Thread/1","doc":"","ref":"beam.html#Thread"},{"type":"type","title":"beam.ThreadedCallbacks/1","doc":"","ref":"beam.html#ThreadedCallbacks"},{"type":"type","title":"beam.env/0","doc":"","ref":"beam.html#env"},{"type":"type","title":"beam.event/0","doc":"","ref":"beam.html#event"},{"type":"type","title":"beam.loader/0","doc":"","ref":"beam.html#loader"},{"type":"type","title":"beam.monitor/0","doc":"","ref":"beam.html#monitor"},{"type":"type","title":"beam.pid/0","doc":"","ref":"beam.html#pid"},{"type":"type","title":"beam.port/0","doc":"","ref":"beam.html#port"},{"type":"type","title":"beam.term/0","doc":"","ref":"beam.html#term"},{"type":"type","title":"beam.tid/0","doc":"","ref":"beam.html#tid"},{"type":"extras","title":"Zigler","doc":"# Zigler\n\nLibrary test status:\n![](https://github.com/ityonemo/zigler/workflows/Elixir%20CI/badge.svg)","ref":"readme.html"},{"type":"extras","title":"Version 0.11.1 warning - Zigler","doc":"Several compile-time niceties are broken, such as line and file translation and\nthere is a compile time warning about multiple function matches.  These will be\naddressed in the 0.12.0 release.","ref":"readme.html#version-0-11-1-warning"},{"type":"extras","title":"Installation: Elixir - Zigler","doc":"#","ref":"readme.html#installation-elixir"},{"type":"extras","title":"Zig dependency - Zigler","doc":"Prior to installing Zigler, you should install zig via `mix zig.get`.  To do so, install the\n`minisign` and `zig.get` mix tasks:\n\n`mix archive.install hex minisign`\n`mix archive.install hex zig_get`\n\nAnd then run `mix zig.get`\n\nIf you're using erlang, please install zig in the following locations:\n\n##","ref":"readme.html#zig-dependency"},{"type":"extras","title":"Linux - Zigler","doc":"`~/.cache/zigler/zig-linux- -0.11.1`\n\n#","ref":"readme.html#linux"},{"type":"extras","title":"Main installation - Zigler","doc":"Zigler is [available in Hex](https://hex.pm/packages/zigler), and the package can be installed\nby adding `zigler` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:zigler, \"~> 0.11.0\", runtime: false}\n  ]\nend\n```","ref":"readme.html#main-installation"},{"type":"extras","title":"Installation: Erlang - Zigler","doc":"Erlang is only supported via rebar3.  You must enable the rebar_mix plugin and \nadd zigler to your deps in rebar3.\n\nNote that erlang support is highly experimental.  Please submit issues if you \nhave difficulty.\n\n```erlang\n{plugins, [rebar_mix]}.\n\n{deps, [{zigler, \"0.11\"}]}.\n\n```","ref":"readme.html#installation-erlang"},{"type":"extras","title":"Documentation - Zigler","doc":"Docs can be found at [https://hexdocs.pm/zigler](https://hexdocs.pm/zigler).","ref":"readme.html#documentation"},{"type":"extras","title":"Currently supported platforms - Zigler","doc":"- Linux\n- FreeBSD (tested, but not subjected to CI)\n- MacOS\n\n- Nerves cross-compilation is supported out of the box.","ref":"readme.html#currently-supported-platforms"},{"type":"extras","title":"Zig Nifs made easy - Zigler","doc":"Wouldn't it be nice if you could make NIFs as easily as you can use the `asm`\nkeyword in C?\n\nThis is now possible, using the magic of Zig.\n\n```elixir\ndefmodule ExampleZig do\n  use Zig, otp_app: :zigler\n  ~Z\"\"\"\n  pub fn example_fun(value1: f64, value2: f64) bool {\n    return value1 > value2;\n  }\n  \"\"\"\nend\n\ntest \"example nifs\" do\n  assert ExampleZig.example_fun(0.8, -0.8)\n  refute ExampleZig.example_fun(0.1, 0.4)\nend\n```\n\nZigler will do automatic type marshalling between Elixir code and Zig code.\nIt will also convert trickier types into types you care about, for example:\n\n```elixir\ndefmodule ZigCollections do\n  use Zig, otp_app: :zigler\n  ~Z\"\"\"\n  pub fn string_count(string: []u8) i64 {\n    return @intCast(string.len);\n  }\n\n  pub fn list_sum(array: []f64) f64 {\n    var sum: f64 = 0.0;\n    for(array) | item | {\n      sum += item;\n    }\n    return sum;\n  }\n  \"\"\"\nend\n\ntest \"type marshalling\" do\n  assert 9 == ZigCollections.string_count(\"hello zig\")\n  assert 6.0 == ZigCollections.list_sum([1.0, 2.0, 3.0])\nend\n```\n\nMemory allocation with zigler is easy!  A standard BEAM allocator is provided for you,\nso any zig code you import will play nice with the BEAM.\n\n```elixir\ndefmodule Allocations do\n  use Zig, otp_app: :zigler\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n\n  pub fn double_atom(env: beam.env, string: []u8) beam.term {\n    var double_string = beam.allocator.alloc(u8, string.len * 2) catch {\n      return beam.raise_enomem(env);\n    };\n\n    defer beam.allocator.free(double_string);\n\n    for (string) | char, i | {\n      double_string[i] = char;\n      double_string[i + string.len] = char;\n    }\n\n    return beam.make_atom(env, double_string);\n  }\n  \"\"\"\nend\n\ntest \"allocations\" do\n  assert :foofoo == Allocations.double_atom(\"foo\")\nend\n```\n\nIt is a goal for Zigler to make using *it* to bind C libraries easier\nthan using C to bind C libraries.  Here is an example:\n\n```elixir\ndefmodule Blas do\n  use Zig,     \n    otp_app: :zigler\n    link_lib: {:system, \"blas\"},\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const blas = @cImport({\n    @cInclude(\"cblas.h\");\n  });\n\n  pub fn blas_axpy(env: beam.env, a: f64, x: []f64, y: []f64) beam.term {\n    if (x.len != y.len) {\n      return beam.raise_function_clause_error(env);\n    }\n\n    blas.cblas_daxpy(@intCast(x.len), a, x.ptr, 1, y.ptr, 1);\n\n    return y;\n  }\n  \"\"\"\nend\n\ntest \"we can use a blas shared library\" do\n  # returns aX+Y\n  assert [11.0, 18.0] == Blas.blas_axpy(3.0, [2.0, 4.0], [5.0, 6.0])\nend\n```\n\n#","ref":"readme.html#zig-nifs-made-easy"},{"type":"extras","title":"Documentation (Elixir-only) - Zigler","doc":"You can document nif functions, local functions, zig structs, variables, and types.\nIf you document a nif function, it will be a part of the module documentation, and\naccessible using the iex `h` method, etc.\n\nExample:\n\n```elixir\ndefmodule Documentation do\n  use Zig, otp_app: :zigler\n  ~Z\"\"\"\n  /// a zero-arity function which returns 47.\n  pub fn zero_arity() i64 {\n    return 47;\n  }\n  \"\"\"\nend\n```\n\n#","ref":"readme.html#documentation-elixir-only"},{"type":"extras","title":"Formatting (Elixir-only) - Zigler","doc":"A mix format plugin is available through the `zigler_format` package.\n[See the installation instructions](https://github.com/v0idpwn/zigler_format#installation)","ref":"readme.html#formatting-elixir-only"},{"type":"extras","title":"Erlang support - Zigler","doc":"Use of Zigler with erlang is possible using parse transforms.  Annotate the zig\ncode into a `zig_code` attribute and pass zigler options (identical to the elixir\noptions) into a `zig_opts` attribute.  Zigler will then create appropriate\nfunctions matching the zig functions.\n\n```erlang\n-module(erlang_zigler_module).\n-compile({parse_transform, zigler}). \n-export([foo/1, foo/0]).\n\n-zig_code(\"\npub fn foo() i32 {\n    return 47;\n}\n\").\n\n-zig_opts([{otp_app, zigler}]).\n\nfoo(X) ->\n    47 + X.\n```","ref":"readme.html#erlang-support"},{"type":"extras","title":"Zigler Principles - Zigler","doc":"1. Make being a good citizen of the BEAM easy.\n2. Use magic, but sparingly, only to prevent errors.\n3. Let the user see behind the curtain.\n4. Let the user opt out of magic.\n5. Magic shouldn't get in the way.","ref":"readme.html#zigler-principles"},{"type":"extras","title":"Using Nifs","doc":"# Using Nifs\n\nNifs are the entrypoint between your BEAM code and your C ABI code.  Zigler \nprovides semantics which are designed to make it easy to write safe NIF code \nwith the safety, memory guarantees that Zig provides.","ref":"1-nifs.html"},{"type":"extras","title":"Preamble - Using Nifs","doc":"Near the top of your module you should use the `use Zig` directive.  This will \nactivate Zigler to seek zig code blocks and convert them to functions.  You \nmust provide the `otp_app` option, which enables Zigler to find a directory to \nplace compilation artifacts (such as libraries) so that they can be shipped \nwith releases.  By default, this will be `/priv/lib`.\n\n> #","ref":"1-nifs.html#preamble"},{"type":"extras","title":"compilation artifacts {: .warning } - Using Nifs","doc":">\n> defaults for compilation artifacts may change in future versions of \n> zigler.\n\nNote that we can ship zig code in *any* module in-place, so they will live \nalongside other functions or even other macro alterations you make to the \nmodule.  In this case, we'll build our code into a module that will be tested \nwith ExUnit.\n\n```elixir\ndefmodule NifGuideTest do\n  use Zig, otp_app: :zigler\n  use ExUnit.Case, async: true\n```","ref":"1-nifs.html#compilation-artifacts-warning"},{"type":"extras","title":"Basic function writing - Using Nifs","doc":"Once zigler has been activated for a module, write `~Z` code anywhere and this \ncode will be assembled into a zig file that will be compiled into the nif \nartifact.  \n\nThen write your desired zig function Zigler will also mount functions with the \n*same name* as the function in the body of the module.\n\n#","ref":"1-nifs.html#basic-function-writing"},{"type":"extras","title":"Example: simple scalar values - Using Nifs","doc":"```elixir\n~Z\"\"\"\npub fn add_one(input: i32) i32 {\n  return input + 1;\n}\n\"\"\"\n\ntest \"add one\" do\n  assert 48 == add_one(47)\nend\n```\n\nNote that Zigler will automatically marshal input and output values across the \nnif boundary.  The following scalar types are accepted by zigler:\n\n- signed integer (`i0`..`i65535`), including non-power-of-two values\n- unsigned integer (`u0`..`u65535`), including non-power-of-two values\n- `usize`, `isize`, architecture-dependent size (roughly `size_t` and \n  `ssize_t` in C)\n- `c_char`, `c_short`, `c_ushort`, `c_int`, `c_uint`, `c_ulong`, \n  `c_longlong`, `c_ulonglong`, which are architecture-dependent\n  integer sizes mostly used for c interop.\n- floats `f16`, `f32`, and `f64`.\n- `bool` (use the atoms `true` or `false` exclusively)\n\n> ##","ref":"1-nifs.html#example-simple-scalar-values"},{"type":"extras","title":"Floating point datatypes {: .info } - Using Nifs","doc":">\n> Floating point datatypes can take the atoms `:infinity`, \n> `:neg_infinity`, and `:NaN`.\n\n> ##","ref":"1-nifs.html#floating-point-datatypes-info"},{"type":"extras","title":"Boolean datatypes {: .info } - Using Nifs","doc":">\n> You must pass boolean datatypes `true` or `false` atoms.\n\nZigler can also marshal list parameters into array, and array-like \ndatatypes:\n\n#","ref":"1-nifs.html#boolean-datatypes-info"},{"type":"extras","title":"Example: Array-like datatypes - Using Nifs","doc":"```elixir\n~Z\"\"\"\npub fn sum(input: []f32) f32 {\n  var total: f32 = 0.0;\n  for (input) | value | { total += value; }\n  return total;\n}\n\"\"\"\n\ntest \"sum\" do\n  assert 6.0 == sum([1.0, 2.0, 3.0])\nend\n```\n\nThe following array-like datatypes are allowed for parameters:\n- arrays `[3]T` (for example).  Note the length is compile-time known.\n- slices `[]T`\n- pointers to arrays `*[3]T` (for example).\n- multipointers `[*]T`\n- sentinel-terminated versions of all of the above.\n- cpointers `[*c]T`\n\n#","ref":"1-nifs.html#example-array-like-datatypes"},{"type":"extras","title":"Example: Array-like datatypes as binaries - Using Nifs","doc":"For all scalar child types, Array-like datatypes may be passed as binaries,\nthus the following code works with no alteration:\n\n```elixir\ntest \"sum, with binary input\" do\n  assert 6.0 == sum( >)\nend\n```\n\nThis also results in a natural interface for treating BEAM binaries as `u8` \narrays or slices.\n\n#","ref":"1-nifs.html#example-array-like-datatypes-as-binaries"},{"type":"extras","title":"Example: Marshalling errors - Using Nifs","doc":"Zigler will generate code that protects you from sending incompatible \ndatatypes to the desired function:\n\n```elixir\ntest \"marshalling error\" do\n  assert_raise ArgumentError, \"\"\"\n  errors were found at the given arguments:\n\n    * 1st argument: \n  \n       expected:  > | list(float | :infinity | :neg_infinity | :NaN) (for `[]f32`)\n       got: `:atom`\n  \"\"\", fn ->\n    sum(:atom)\n  end\nend\n```\n\nFor more on marshalling collection datatypes, see [`collections`](collections.html).","ref":"1-nifs.html#example-marshalling-errors"},{"type":"extras","title":"Marshalling types manually - Using Nifs","doc":"You may also manually marshal types into and out of the beam by using the\n[`beam.term`](beam.html#term) datatype.  To do so, you must first import \nthe [`beam`](beam.html) package.  The `beam.term` type is an opaque, wrapped \ndatatype that ensures safe manipulation of terms as a token in your zig code.\n\nYou will also need [`beam.env`](beam.html#env) environment variable to reference\nthe execution environmet of your nif, to box and unbox data from the beam terms.\n\n```elixir\n~Z\"\"\"\nconst beam = @import(\"beam\");\n\npub fn manual_addone(env: beam.env, value_term: beam.term) !beam.term {\n    const value = try beam.get(i32, env, value_term, .{});\n    return beam.make(env, value + 1, .{});\n}\n\"\"\"\n\ntest \"manual marshalling\" do\n  assert 48 == manual_addone(47)\nend\n```\n\n#","ref":"1-nifs.html#marshalling-types-manually"},{"type":"extras","title":"A few notes on the above code. - Using Nifs","doc":"- to marshal a value out of a `beam.term` and into a zig static type,\n  use [`beam.get`](beam.html#get).  This is a failable function, and in\n  this case we hoist the failure in the function return.\n- to marshal a value into a `beam.term` from a zig static type, use\n  [`beam.make`](beam.html#make).  This function *does not fail*.\n- for more information on the final options parameter of `get` and\n  `make`, see their respective documentation.\n- zigler will translate the hoisted marshalling failures into BEAM \n  exceptions.\n- zigler automatically ignores the `beam.env` parameter in the first\n  position, and assigns the correct arity (1).  Compilation will\n  fail if a `beam.env` parameter is present in any other location.","ref":"1-nifs.html#a-few-notes-on-the-above-code"},{"type":"extras","title":"Collection Datatypes","doc":"# Collection Datatypes","ref":"2-collections.html"},{"type":"extras","title":"Returning array-like datatypes - Collection Datatypes","doc":"For array-like datatypes, we saw in [Using Nifs](1-nifs.html#example-array-like-datatypes) \nhow they can take both lists or binaries as inputs.  However, when returning\nstatically-typed data from zig, a choice needs to be made as to whether to\nreturn lists or binaries.\n\n#","ref":"2-collections.html#returning-array-like-datatypes"},{"type":"extras","title":"Arrays - Collection Datatypes","doc":"Arrays are the simplest array-like datatype to return.\n\n```elixir\n~Z\"\"\"\npub fn return_array(input: f32) [3]f32 {\n  var result: [3]f32 = undefined;\n\n  // set each item in the array:\n  for (&result, 0..) |*item, index| {\n    item.* = input + @as(f32, @floatFromInt(index));\n  }\n  return result;\n}\n\"\"\"\n\ntest \"returning an array\" do\n  assert [47.0, 48.0, 49.0] == return_array(47.0)\nend\n```\n\n#","ref":"2-collections.html#arrays"},{"type":"extras","title":"Slices - Collection Datatypes","doc":"Slices can also be returned.  Note that in many cases, returning\na slice might need an [allocation strategy](3-allocators.html)\n\n```elixir\n~Z\"\"\"\npub fn return_slice(input: []f32) []f32 {\n  // set each item in the array:\n  for (input, 0..) |*item, index| {\n    item.* = item.* + @as(f32, @floatFromInt(index));\n  }\n  return input;\n}\n\"\"\"\n\ntest \"returning a slice\" do\n  assert [47.0, 48.0, 49.0] == return_slice([47.0, 47.0, 47.0])\nend\n```\n\n> #","ref":"2-collections.html#slices"},{"type":"extras","title":"u8 array-likes output as binary {: .info } - Collection Datatypes","doc":">\n> u8 array-like datatypes are marshalled into binary \n> by default instead of list.\n\n```elixir\n~Z\"\"\"\npub fn return_u8_array() [3]u8 {\n    const result: [3]u8 = .{97, 98, 99};\n    return result;\n}\n\"\"\"\n\ntest \"u8 datatypes are returned as binary\" do\n  assert \"abc\" == return_u8_array()\nend \n```\n\n#","ref":"2-collections.html#u8-array-likes-output-as-binary-info"},{"type":"extras","title":"Selecting output type - Collection Datatypes","doc":"It's also possible to return these collections as binaries,\nhowever, in order to do so you will have to marshal manually.\nFor datatypes that are more than 1 byte, be aware that the endianness\nof the resulting data is `native`.\n\n```elixir\n~Z\"\"\"\nconst beam = @import(\"beam\");\n\npub fn return_slice_binary(env: beam.env, input: []f32) beam.term {\n  // set each item in the array:\n  for (input, 0..) |*item, index| {\n    item.* = item.* + @as(f32, @floatFromInt(index));\n  }\n  return beam.make(env, input, .{.output_type = .binary});\n}\n\"\"\"\n\ntest \"returning a slice as binary\" do\n  assert  > == return_slice_binary([47.0, 47.0, 47.0])\nend\n```\n\nConversely, for u8 array-like datatypes, selecting `.output_type = .list` will\nresult in outputting a charlist.\n\nYou can also automatically marshal as binary by using\n[nif options](4-nif_options.html#binary-output)\n\n#","ref":"2-collections.html#selecting-output-type"},{"type":"extras","title":"Full list of qualified array-like return types - Collection Datatypes","doc":"The central challenge of exporting array-like data to the BEAM is that length\ninformation may not be known.  In the case of arrays and slices, length \nis either comptime or runtime known.  For other data types, the scope of\ndatatypes accepted must be limited:\n\n- array of any type (`[_]T`)\n- single pointers to an array (`*[_]T`)\n- slices (`[]T`)\n- sentinel-terminated forms of the above (`[_:0]T` or `[:0]T`)\n- sentinel-terminated many-item-pointer (`[*:0]T`)\n- cpointer to u8 (`[*c]u8`).  This will assume the cpointer is null-terminated,\n  if it can't be considered `[*:0]u8` the behaviour is undefined.\n- cpointer to a pointer (`[*c]?*T`).  This will assume cpointer is \n  null-terminated, if it can't be considered `[*:null]?*T` then the\n  behaviour is undefined.","ref":"2-collections.html#full-list-of-qualified-array-like-return-types"},{"type":"extras","title":"Passing and returning enums - Collection Datatypes","doc":"Enums are collections of integer values that are given special identifier\nstatus in the zig programming language.  At compile-time, it's possible\nto get reflection on the string representation of those identifiers.  \nZigler thus is enabled to use enums as a representation of atoms coming \nfrom or going to the BEAM.\n\n#","ref":"2-collections.html#passing-and-returning-enums"},{"type":"extras","title":"Enums as atoms - Collection Datatypes","doc":"Enums can be created by referring to them by the atom that corresponds\nto their value:\n\n```elixir\n~Z\"\"\"\nconst EnumType = enum(u8) {\n  foo, \n  bar = 47\n};\n\npub fn flip(input: EnumType) EnumType {\n  return switch (input) {\n    .foo => .bar,\n    .bar => .foo\n  };\n}\n\"\"\"\n\ntest \"flipping enums\" do\n  assert :bar = flip(:foo)\n  assert :foo = flip(:bar)\nend\n```\n\n#","ref":"2-collections.html#enums-as-atoms"},{"type":"extras","title":"Enums as integers - Collection Datatypes","doc":"Functions taking an integer type can also by passed the associated integer \nvalue in the place of the atom.\n\n```elixir\ntest \"enums passed as integer\" do\n  assert :foo = flip(47)\nend\n```\n\n#","ref":"2-collections.html#enums-as-integers"},{"type":"extras","title":"Enum literals - Collection Datatypes","doc":"Enum literals can be converted to atoms using [`beam.make`](beam.html#make).\n\n```elixir\n~Z\"\"\"\npub fn make_literal(env: beam.env) beam.term {\n  return beam.make(env, .some_new_literal, .{});\n}\n\"\"\"\n\ntest \"enum literals\" do\n  assert :some_new_literal = make_literal()\nend\n```\n\nThis is especially useful for emitting `:ok` or `:error` tuples.\n\n> #","ref":"2-collections.html#enum-literals"},{"type":"extras","title":"error atom {: .info } - Collection Datatypes","doc":">\n> `error` is a reserved word in Zig, so to create error atom you must use\n> builtin syntax; the error enum literal is represented `.@\"error\"`.","ref":"2-collections.html#error-atom-info"},{"type":"extras","title":"Passing and returning structs - Collection Datatypes","doc":"#","ref":"2-collections.html#passing-and-returning-structs"},{"type":"extras","title":"Structs are atom-keyed maps - Collection Datatypes","doc":"Most structs are interpreted as atom-keyed maps.  Consider the following code:\n\n```elixir\n~Z\"\"\"\npub const Point2D = struct{ x: i32, y: i32 };\n\npub fn reflect(input: Point2D) Point2D {\n  return .{.x = input.y, .y = input.x};\n}\n\"\"\"\n\ntest \"structs\" do\n  assert %{x: 48, y: 47} == reflect(%{x: 47, y: 48})\nend\n```\n\n> #","ref":"2-collections.html#structs-are-atom-keyed-maps"},{"type":"extras","title":"Structs in parameters and returns {: .info } - Collection Datatypes","doc":">\n> for a struct type to be used in parameters and returns,\n> it must be exported as `pub` in the module interface\n\n#","ref":"2-collections.html#structs-in-parameters-and-returns-info"},{"type":"extras","title":"Anonymous structs can be returned - Collection Datatypes","doc":"It's possible to return anonymous structs as well, using \n[`beam.make`](beam.html#make).\n\n```elixir\n~Z\"\"\"\npub fn anonymous_struct(env: beam.env) beam.term {\n    return beam.make(env, .{.foo = .bar}, .{});\n}\n\"\"\"\n\ntest \"anonymous structs\" do\n  assert %{foo: :bar} == anonymous_struct()\nend\n```\n\n#","ref":"2-collections.html#anonymous-structs-can-be-returned"},{"type":"extras","title":"Zig tuples are structs. - Collection Datatypes","doc":"Tuples in zig are structs (with hidden integer-valued keys).\nZigler takes advantage of this and allows you to construct\nBEAM tuples using zig tuples, when passed to [`beam.make`](beam.html#make).\n\n```elixir\n~Z\"\"\"\npub fn tuple(env: beam.env) beam.term {\n    return beam.make(env, .{.ok, 47}, .{});\n}\n\"\"\"\n\ntest \"tuples\" do\n  assert {:ok, 47} == tuple()\nend\n```\n\n#","ref":"2-collections.html#zig-tuples-are-structs"},{"type":"extras","title":"Packed or Extern structs. - Collection Datatypes","doc":"Packed or Extern structs can be passed as binaries.\n\n```elixir\n~Z\"\"\"\npub const Packed = packed struct {x: u4, y: u4};\n\npub const Extern = extern struct {x: u16, y: u16};\n\npub fn diff_packed(value: Packed) u8 {\n    return value.x - value.y;\n}\n\npub fn diff_extern(value: Extern) u16 {\n    return value.x - value.y;\n}\n\"\"\"\n\ntest \"packed and extern structs as struct\" do\n  assert 2 = diff_packed(%{x: 7, y: 5})\n  assert 5 = diff_extern(%{x: 47, y: 42})\nend\n\ntest \"packed and extern structs as binary\" do\n  assert 2 = diff_packed( >)\n  assert 5 = diff_extern( >)\nend\n```\n\n> #","ref":"2-collections.html#packed-or-extern-structs"},{"type":"extras","title":"Packed And Extern Endianness {: .warning } - Collection Datatypes","doc":">\n> Be careful about the endianness of packed and extern structs!\n\n#","ref":"2-collections.html#packed-and-extern-endianness-warning"},{"type":"extras","title":"Pointers to structs. - Collection Datatypes","doc":"Pointers to structs can also be used to marshal data in and out.  This\nis enabled under the assumption that you might want to use the struct\nin a mutable fashion.\n\n```elixir\n~Z\"\"\"\npub fn swap_pointer(value: *Point2D) *Point2D {\n    const temp = value.x;\n    value.x = value.y;\n    value.y = temp;\n\n    return value;\n}\n\"\"\"\n\ntest \"pointer to structs\" do\n  assert %{x: 47, y: 50} == swap_pointer(%{x: 50, y: 47})\nend\n```","ref":"2-collections.html#pointers-to-structs"},{"type":"extras","title":"Nested collections - Collection Datatypes","doc":"The process of marshalling parameters and returns also works with \nnested array-like and struct data.\n\n#","ref":"2-collections.html#nested-collections"},{"type":"extras","title":"Arraylike of arraylike - Collection Datatypes","doc":"```elixir\n~Z\"\"\"\npub fn array_of_array_sum(a_of_a: [][]u64) u64 {\n    var sum: u64 = 0;\n    for (a_of_a) |inner_array| {\n        for (inner_array) |value| {\n            sum += value;\n        }\n    }\n    return sum;\n}\n\"\"\"\n\ntest \"array of array\" do\n  assert 21 = array_of_array_sum([[1, 2, 3], [4], [5, 6]])\nend\n```\n\n#","ref":"2-collections.html#arraylike-of-arraylike"},{"type":"extras","title":"Structs of structs - Collection Datatypes","doc":"```elixir\n~Z\"\"\"\npub const Arrow = struct {\n    head: Point2D,\n    tail: Point2D\n};\n\npub fn reflect_reverse_arrow(arrow: Arrow) Arrow {\n    return .{\n      .head = reflect(arrow.tail), \n      .tail = reflect(arrow.head)\n    };\n}\n\"\"\"\n\ntest \"structs of structs\" do\n  assert %{\n    head: %{x: 1, y: 2},\n    tail: %{x: 3, y: 4}\n  } == reflect_reverse_arrow(%{\n    head: %{x: 4, y: 3},\n    tail: %{x: 2, y: 1}\n  })\nend\n```\n\n> ##","ref":"2-collections.html#structs-of-structs"},{"type":"extras","title":"Deep Argument Errors {: .info } - Collection Datatypes","doc":">\n> Argument errors for deeply nested structs will help you understand\n> where your arguments failed to serialize:\n\n```elixir\n# note: skipped because map key order is nondeterministic before 1.15\n@tag [skip: Version.compare(System.version(), \"1.15.0\") == :lt]\ntest \"argument errors\" do\n  assert_raise ArgumentError, \"\"\"\n  errors were found at the given arguments:\n  \n    * 1st argument: \n  \n       expected: map | keyword (for `Arrow`)\n       got: `%{head: %{x: 4, y: 3}, tail: %{x: 2, y: 1.0}}`\n       in field `:tail`:\n       | expected: map | keyword (for `Point2D`)\n       | got: `%{x: 2, y: 1.0}`\n       | in field `:y`:\n       | | expected: integer (for `i32`)\n       | | got: `1.0`\n  \"\"\", fn ->\n    reflect_reverse_arrow(%{\n      head: %{x: 4, y: 3},\n      tail: %{x: 2, y: 1.0}\n    })\n  end\nend\n```\n\n#","ref":"2-collections.html#deep-argument-errors-info"},{"type":"extras","title":"Arraylike of structs - Collection Datatypes","doc":"```elixir\n~Z\"\"\"\npub fn sum_points(points: []Point2D) Point2D {\n    var result: Point2D = .{.x = 0, .y = 0};\n    for (points) |point| {\n        result.x += point.x;\n        result.y += point.y;\n    }\n    return result;\n}\n\"\"\"\n\ntest \"array of struct\" do\n  assert %{x: 9, y: 12} = sum_points([%{x: 1, y: 2}, %{x: 3, y: 4}, %{x: 5, y: 6}])\nend\n```\n\n#","ref":"2-collections.html#arraylike-of-structs"},{"type":"extras","title":"Structs of arraylikes - Collection Datatypes","doc":"```elixir\n~Z\"\"\"\npub const PointSOA = struct{\n  x: []u16,\n  y: []u16\n};\n\npub fn sum_point_soa(points: PointSOA) Point2D {\n    var result: Point2D = .{.x = 0, .y = 0};\n    for (points.x) |x| {\n        result.x += x;\n    }\n    for (points.y) |y| {\n        result.y += y;\n    }\n    return result;\n}\n\"\"\"\n\ntest \"struct of array\" do\n  assert %{x: 9, y: 12} = sum_point_soa(%{x: [1, 3, 5], y: [2, 4, 6]})\nend\n```\n\n>","ref":"2-collections.html#structs-of-arraylikes"},{"type":"extras","title":"Interaction with allocators {: .warning } - Collection Datatypes","doc":">\n> if you directly return a datatype that was allocated, it won't be \n> properly cleaned up.  However, it can be properly cleaned up by\n> manually deferring its cleanup after calling [`beam.make`](beam.html#make).\n>\n> Cleanup routines in [nif options](4-nif_options.html) will be introduced\n> in a future release, which will enable protection from these sorts of leaks.\n>\n> For more on allocators, see [allocators](3-allocators.html)\n\n```elixir\n~Z\"\"\"\npub fn leaks() !*Point2D {\n    var point = try beam.allocator.create(Point2D);\n\n    point.x = 47;\n    point.y = 50;\n\n    return point;\n}\n\npub fn no_leak(env: beam.env) !beam.term {\n    var point = try beam.allocator.create(Point2D);\n    defer beam.allocator.destroy(point);\n\n    point.x = 47;\n    point.y = 50;\n\n    return beam.make(env, point, .{});\n}\n\"\"\"\n\ntest \"both leaky and non-leaky struct returns work\" do\n  assert %{x: 47, y: 50} == leaks()\n  assert %{x: 47, y: 50} == no_leak()\nend\n```","ref":"2-collections.html#interaction-with-allocators-warning"},{"type":"extras","title":"Allocation using zigler","doc":"# Allocation using zigler\n\nZig the language has no offically supported allocator, and the standard library\ndatastructures are all allocator-agnostic.  \n\nZigler ships with three primary allocators, though you can certainly build\nallocator strategies *on top* of those allocators.","ref":"3-allocators.html"},{"type":"extras","title":"Raw Beam Allocator - Allocation using zigler","doc":"The first allocator is `raw_allocator`.  This allocator wraps the \n[nif allocator](https://www.erlang.org/doc/man/erl_nif.html#enif_alloc) \nprovided by the BEAM.  You should use this allocator over `malloc` because it\noften saves a syscall by using existing preallocated memory pools, because\nit allows the VM to track how much memory your NIF is using, and possibly\ngives better memory placement to avoid cache misses in your execution\nthread.\n\n```elixir\n~Z\"\"\"\nconst beam = @import(\"beam\");\n\npub fn allocate_raw(env: beam.env, count: usize) !beam.term {\n    var slice = try beam.raw_allocator.alloc(u16, count);\n    defer beam.raw_allocator.free(slice);\n\n    for (slice, 0..) |*entry, index| {\n        entry.* = @intCast(index);\n    }\n    return beam.make(env, slice, .{});\n}\n\"\"\"\n\ntest \"raw allocator\" do\n  assert [0, 1, 2] = allocate_raw(3)\nend\n```\n\n> #","ref":"3-allocators.html#raw-beam-allocator"},{"type":"extras","title":"raw allocator limitations {: .warning } - Allocation using zigler","doc":">\n> because the raw allocator directly wraps the beam allocator, according to\n> the documentation:\n>\n> The returned pointer is suitably aligned for any built-in type that \n> fit (sic) in the allocated memory.\n>\n> attempting to allocate memory aligned to a larger size (e.g. page-aligned \n> allocation) will fail using this allocator.\n\n#","ref":"3-allocators.html#raw-allocator-limitations-warning"},{"type":"extras","title":"Tracking memory. - Allocation using zigler","doc":"> #","ref":"3-allocators.html#tracking-memory"},{"type":"extras","title":"information in hidden globals {: .warning } - Allocation using zigler","doc":">\n> Generally storing information in hidden globals is not a good idea.  Here\n> it is done to illustrate the memory usage.  A better strategy would be to\n> use [resources](5-resources.html)\n\n```elixir\n~Z\"\"\"\nvar global_zigler: []u8 = undefined;\n\npub fn zigler_alloc() !void {\n    global_zigler = try beam.raw_allocator.alloc(u8, 1_000_000);\n}\n\npub fn zigler_free() void {\n    beam.raw_allocator.free(global_zigler);\n}\n\nconst c_stdlib = @cImport(@cInclude(\"stdlib.h\"));\n\nvar global_cstd: [*c]u8 = undefined;\npub fn c_malloc() void {\n    global_cstd = @ptrCast(c_stdlib.malloc(1_000_000));\n}\n\npub fn c_free() void {\n    c_stdlib.free(global_cstd);\n}\n\"\"\"\n\ntest \"zigler memory is tracked\" do\n  Process.sleep(100)\n  start = :erlang.memory[:total]\n  zigler_alloc()\n  assert :erlang.memory[:total] - start >= 1_000_000\n  zigler_free()\nend\n\ntest \"malloc memory is not tracked\" do\n  Process.sleep(100)\n  start = :erlang.memory[:total]\n  c_malloc()\n  assert :erlang.memory[:total] - start <= 1_000_000\n  c_free()\nend\n```","ref":"3-allocators.html#information-in-hidden-globals-warning"},{"type":"extras","title":"Large Allocator - Allocation using zigler","doc":"Zigler provides a `large allocator` which allows you to allocate memory ranges\nthat have a higher alignment than the maximum alignment for builtin types.\nNote that using this allocator comes with a memory penalty.\n\n```elixir\n~Z\"\"\"\npub fn allocate_large_aligned(count: usize) !usize {\n    const page = try beam.large_allocator.allocWithOptions(u8, count, 4096, null);\n    defer beam.large_allocator.free(page);\n\n    return @intFromPtr(page.ptr);\n}\n\"\"\"\n\ntest \"aligned allocation\" do\n  assert 0 = rem(allocate_large_aligned(3), 4096)\nend\n```","ref":"3-allocators.html#large-allocator"},{"type":"extras","title":"General Purpose Allocator - Allocation using zigler","doc":"Zigler provides a version of the zig standard library's \n`GeneralPurposeAllocator` which is built on top of the large allocator.  Two \nadvantages of using the general purpose allocator include optimized memory \nlayouts for mixed allocation sizes and the ability to track memory leaks.\n\nThe state of the general purpose allocator is accessible using\n`beam.allocator_.general_purpose_allocator_instance`\n\n```elixir\n~Z\"\"\"\npub fn leaks() !bool {\n    const memory = try beam.general_purpose_allocator.alloc(u8, 8);\n    defer beam.general_purpose_allocator.free(memory);\n\n    // note that we haven't freed it yet, that happens on deferral,\n    // which lands after the return call.\n\n    return beam.allocator_.general_purpose_allocator_instance.detectLeaks();\n}\n\npub fn noleak() !bool {\n    const memory = try beam.general_purpose_allocator.alloc(u8, 8);\n    beam.general_purpose_allocator.free(memory);\n    return beam.allocator_.general_purpose_allocator_instance.detectLeaks();\n}\n\"\"\"\n\ntest \"leak checks with general purpose allocator\" do\n  assert leaks()\n  refute noleak()\nend\n```","ref":"3-allocators.html#general-purpose-allocator"},{"type":"extras","title":"beam.allocator - Allocation using zigler","doc":"Zigler provides a `threadlocal` variable: `beam.allocator`.  This is set on entry\ninto the nif and defaults to `beam.raw_allocator`\n\n```elixir\n~Z\"\"\"\npub fn basic(env: beam.env) !beam.term {\n    const slice = try beam.allocator.alloc(u16, 4);\n    defer beam.allocator.free(slice);\n\n    for (slice, 0..) |*item, index| {\n        item.* = @intCast(index);\n    }\n\n    return beam.make(env, slice, .{});\n}\n\"\"\"\n\ntest \"leak checks with allocator\" do\n  assert [0, 1, 2, 3] = basic()\nend\n```\n\n> #","ref":"3-allocators.html#beam-allocator"},{"type":"extras","title":"Raw nifs {: .warning } - Allocation using zigler","doc":">\n> For raw nifs, `beam.allocator` is not set, and may retain a value from an\n> arbitrary previous nif invocation.  Consider usage of `beam.allocator` in a\n> raw nif to be undefined unless it is set in the raw nif.","ref":"3-allocators.html#raw-nifs-warning"},{"type":"extras","title":"Custom allocators - Allocation using zigler","doc":"Because zigler's allocators conform to zig's allocator interface, you can use\nany composed allocator in the standard library or any composable allocator \nfrom an imported zig package.\n\n```elixir\n~Z\"\"\"\nconst std = @import(\"std\");\n\npub fn with_arena(env: beam.env) !beam.term {\n    var arena = std.heap.ArenaAllocator.init(beam.allocator);\n    defer arena.deinit();\n\n    const allocator = arena.allocator();\n\n    const slice = try allocator.alloc(u16, 4);\n    defer allocator.free(slice);\n\n    for (slice, 0..) |*item, index| {\n        item.* = @intCast(index);\n    }\n\n    return beam.make(env, slice, .{});\n}\n\"\"\"\n\ntest \"arena allocator\" do\n  assert [0, 1, 2, 3] == with_arena()\nend\n```","ref":"3-allocators.html#custom-allocators"},{"type":"extras","title":"Nif options","doc":"# Nif options\n\nZigler gives you several ways to configure compilation options for nifs.  These\noptions are part of the `use Zig` directive, in the keyword options under the \nkey `:nifs`.  This key itself points to a keyword list where the keys are the\nnames of the nif functions and the values are a keyword list of options that\napply to that function.\n\nThis guide shows you all of the nif options except for options related to \n[Resources](5-resources.html), [C integration](6-c_integration.html), or\n[Concurrency](7-concurrency.html).","ref":"4-nif_options.html"},{"type":"extras","title":"Automatic options (elixir) - Nif options","doc":"To declare that functions should have their options automatically determined,\nuse `...` in the nifs parameter list.  Nifs which sholud have manually decided\noptions should come after the `...` as a keyword list.  If all options are\nautomatically determined, then omitting the `:nif` keyword completely is valid.\n\n```elixir\ndefmodule AutomaticOptions do\n  use Zig, \n    otp_app: :zigler,\n    nifs: [...]\n\n  ~Z\"\"\"\n  pub fn noop() void {}\n  \"\"\"\nend\n```","ref":"4-nif_options.html#automatic-options-elixir"},{"type":"extras","title":"Automatic options (erlang) - Nif options","doc":"Erlang cannot interpret the `...` AST in elixir, so you must use `[auto]`\natom in the nifs options instead.\n\n```erlang\n-zig_opts([{nifs, [auto]}])\n```","ref":"4-nif_options.html#automatic-options-erlang"},{"type":"extras","title":"Raw calls - Nif options","doc":"Raw calls can be signified with the `raw` option; this option takes the arity\nof the function as the value associated with the key.\n\n> #","ref":"4-nif_options.html#raw-calls"},{"type":"extras","title":"multiple arities {: .info } - Nif options","doc":">\n> Currently, multiple arities are not supported, but multiple arities will be\n> supported in a future release.\n\nThe normal header for a BEAM nif is as follows:\n\n`static ERL_NIF_TERM hello(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])`\n\nIn order to support usage of BEAM nifs in the most flexible fashion, you may\nwrite your nifs as `raw` nifs, which looks as follows:\n\n```elixir\ndefmodule RawCallTest do\n  use ExUnit.Case, async: true\n\n  use Zig, \n    otp_app: :zigler,\n    nifs: [\n        raw_call_beam: [raw: 1],\n        raw_call_erl_nif: [raw: 1]\n    ]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const e = @import(\"erl_nif\");\n\n  pub fn raw_call_beam(env: beam.env, count: c_int, list: [*]const beam.term) beam.term {\n      return beam.make(env, .{.count = count, .item = list[0]}, .{});\n  }\n\n  pub fn raw_call_erl_nif(env: beam.env, count: c_int, list: [*]const e.ErlNifTerm) e.ErlNifTerm {\n      return beam.make(env, .{.count = count, .item = beam.term{.v = list[0]}}, .{}).v;\n  }\n  \"\"\"\n\n  test \"raw call with beam package\" do\n    assert %{count: 1, item: {:foo, \"bar\"}} = raw_call_beam({:foo, \"bar\"})\n  end\n\n  test \"raw call with erl_nif package\" do\n    assert %{count: 1, item: {:foo, \"bar\"}} = raw_call_erl_nif({:foo, \"bar\"})\n  end\nend\n```\n\nNote that either the `beam.term` or the `e.ErlNifTerm` forms can be used.","ref":"4-nif_options.html#multiple-arities-info"},{"type":"extras","title":"Return type - Nif options","doc":"In [Collections](2-collections.html) we saw how certain collection types could \nbe manually marshalled into alternative representations using \n[`beam.make`](beam.html#make).  This can be handled as a nif configuration as \nfollows.  The advantage to doing it this way is that the typespec for the\nfunction will correctly reflect the return type.\n\n```elixir\ndefmodule ReturnTypeTest do\n  use ExUnit.Case, async: true\n\n  use Zig, \n    otp_app: :zigler,\n    nifs: [\n        returns_binary: [return: :binary],\n        returns_charlist: [return: :list]\n    ]\n\n  ~Z\"\"\"\n  pub fn returns_binary() [3]u16 {\n      return [3]u16{47, 48, 49};\n  }\n\n  pub fn returns_charlist() []const u8 {\n      return \"Hello world!\";\n  }\n  \"\"\"\n\n  test \"returns binary\" do\n    assert  > = returns_binary()\n  end\n\n  test \"returns charlist\" do\n    assert ~C'Hello world!' = returns_charlist()\n  end\nend\n```","ref":"4-nif_options.html#return-type"},{"type":"extras","title":"Alias - Nif options","doc":"It's possible to create a new function which is an alias of another function.\nThis is how it's done:\n\n```elixir\ndefmodule AliasTest do\n  use ExUnit.Case, async: true\n\n  use Zig, \n    otp_app: :zigler,\n    nifs: [\n      ...,\n      new_function: [alias: :old_function]\n    ]\n\n  ~Z\"\"\"\n  pub fn old_function() u32 {\n      return 47;\n  }\n  \"\"\"\n\n  test \"both main and alias functions work\" do\n    assert 47 == old_function()\n    assert 47 == new_function()\n  end\nend\n```","ref":"4-nif_options.html#alias"},{"type":"extras","title":"Args options - Nif options","doc":"Arguments can also take options, using `args: [...]`","ref":"4-nif_options.html#args-options"},{"type":"extras","title":"Noclean - Nif options","doc":"If you want to disable automatic allocator cleanup of datatypes, you can do so\neither in the `:return` or `:args` section by including the `:noclean` option.  \nThis is most useful if your args or return data are going to be persisted \nbeyond the lifetime of the nif call, though doing this in many cases is not \nrecommended.\n\nThis flag can be stacked with previous options, for example:  \n`return: [:noclean, :binary]`.","ref":"4-nif_options.html#noclean"},{"type":"extras","title":"Leak Check - Nif options","doc":"It's possible to wrap each function call in its own instance of \n[`beam.general_purpose_allocator`](beam.html#general_purpose_allocator)\nbound into the [`beam.allocator`](beam.html#allocator) threadlocal variable.\nIf you tag your nif as `leak_check`, it will check that `beam.allocator` has\ncleared all of its contents at the end of the function call, and if that hasn't \nhappened, it raises.\n\n>","ref":"4-nif_options.html#leak-check"},{"type":"extras","title":"leak check warning {: .warning } - Nif options","doc":">\n> leak check doesn't seem to be working in 0.11.0 and will return in 0.11.1\n\n```elixir\ndefmodule LeakCheckTest do\n  use ExUnit.Case, async: true\n\n  use Zig,\n    otp_app: :zigler,\n    nifs: [check_me: [leak_check: true]]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  pub fn check_me() !void {\n      _ = try beam.allocator.create(u8);\n  }\n  \"\"\"\n\n  @tag :skip\n  test \"leak check\" do\n    assert_raise RuntimeError, \"memory leak detected in function `check_me/0`\", fn ->\n      check_me()\n    end\n  end\nend\n```\n\n`leak_check` can also be applied to all nifs in the module:\n\n```elixir\ndefmodule LeakCheckAllTest do\n  use ExUnit.Case, async: true\n\n  use Zig,\n    otp_app: :zigler,\n    leak_check: true\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  pub fn check_me() !void {\n      _ = try beam.allocator.create(u8);\n  }\n  \"\"\"\n\n  @tag :skip\n  test \"leak check\" do\n    assert_raise RuntimeError, \"memory leak detected in function `check_me/0`\", fn ->\n      check_me()\n    end\n  end\nend\n```\n\n> #","ref":"4-nif_options.html#leak-check-warning-warning"},{"type":"extras","title":"conditional leak checks {: .warning } - Nif options","doc":">\n> It's not currently possible to make conditional leak checks, but this will be\n> fixed in the future.","ref":"4-nif_options.html#conditional-leak-checks-warning"},{"type":"extras","title":"Typespec override (Elixir-only) - Nif options","doc":"Typespecs can be overridden by using the `spec` option, this syntax should \nmatch that of a normal typespec.  Note that overriding the autogenerated \ntypespec is the only way to specify the type of function which uses \n[`beam.term`](beam.html#term) in its arguments or return.\n\nFor example:\n\n```elixir\ndefmodule Override do\n  use Zig, \n    otp_app: :zigler,\n    nifs: [typespec_override: [spec: (integer -> integer)]]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  pub fn typespec_override(env: beam.env, term: beam.term) !beam.term {\n      const input = try beam.get(u32, env, term, .{});\n      return beam.make(env, input + 1, .{});\n  }\n  \"\"\"\nend\n```","ref":"4-nif_options.html#typespec-override-elixir-only"},{"type":"extras","title":"Disable documentation - Nif options","doc":"Documentation can be disabled with the `docs: false` option.\n\n```elixir\ndefmodule DisableDoc do\n  use Zig, \n    otp_app: :zigler,\n    nifs: [nodocs: [docs: false]]\n\n  ~Z\"\"\"\n  pub fn nodocs() void {}\n  \"\"\"\nend\n```","ref":"4-nif_options.html#disable-documentation"},{"type":"extras","title":"Ignore - Nif options","doc":"Public functions can be ignored and *not* converted into nifs by filling out \nthe `:ignore` option in `use Zig` directive.\n\n```elixir\ndefmodule IgnoreTest do\n  use ExUnit.Case, async: true\n\n  use Zig, \n    otp_app: :zigler,\n    ignore: [:ignored]\n\n  ~Z\"\"\"\n  pub fn ignored(number: u32) u32 {\n      return number + 1;\n  }\n\n  pub fn available(number: u32) u32 {\n      return ignored(number);\n  }\n  \"\"\"\n\n  test \"available function works\" do\n    assert 48 = available(47)\n  end\n\n  test \"ignored function is not available\" do\n    refute function_exported?(__MODULE__, :ignored, 0)\n  end\nend\n#module\n```","ref":"4-nif_options.html#ignore"},{"type":"extras","title":"Resources","doc":"# Resources\n\nResources are datatypes which are managed by the BEAM reference-counted garbage\ncollector.  If you are passing data between function calls, generally it is \nbest practice to pass them as a resource instead of as a pointer or a global\nvariable.\n\nFor documentation of how resources work in general, see the erlang \n[documentation](https://www.erlang.org/doc/man/erl_nif.html#resource_objects)\non the topic.  Note that the conveniences in [beam.Resource](beam.html#Resource)\ngeneric type exist to make operations type-safe.\n\n> #","ref":"5-resources.html"},{"type":"extras","title":"Resources are references {: .info } - Resources","doc":">\n> In managed language environments resources are passed as `t:reference/0` \n> tokens.\n\nUnder the hood the BEAM allocates a chunk of memory on resource creation, and \nthis memory is cleared when the garbage collector is triggered.  The BEAM also \nallows you to add in trigger hooks for when this event happens, so custom\ncleanup can be tied into these GC events.\n\n> #","ref":"5-resources.html#resources-are-references-info"},{"type":"extras","title":"Passing references between modules {: .warning } - Resources","doc":">\n> This is not currently supported by Zigler, but support is planned.","ref":"5-resources.html#passing-references-between-modules-warning"},{"type":"extras","title":"Declaring a resource - Resources","doc":"In order to use a resource, you must do at a minimum three things:\n\n1. declare the resource in your `use Zig` directive, under the `:resource`\n  key.  This is a list of atoms, which match the name of the resource type.\n2. declare the wrapped type.  This is the type of the data that is placed in \n  the memory space of the resource.\n3. use [`beam.Resource`](beam.html#Resource) to declare the resource type.\n  note that this resource type must be `pub`.\n\n```elixir\ndefmodule ResourceTest do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler,\n    resources: [\n      :StructResource,\n      :PointerResource\n    ]\n\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const root = @import(\"root\");\n\n  const MyStruct = struct {\n      payload: u64\n  };\n\n  pub const StructResource = beam.Resource(MyStruct, root, .{});\n  \"\"\"\n```","ref":"5-resources.html#declaring-a-resource"},{"type":"extras","title":"Using resources in functions - Resources","doc":"```elixir\n~Z\"\"\"\npub fn create_resource_term(env: beam.env, number: u64) !beam.term {\n    const res = try StructResource.create(.{.payload = number}, .{});\n    return beam.make(env, res, .{});\n}\n\npub fn retrieve_resource_term(env: beam.env, term: beam.term) !u64 {\n    const res = try beam.get(StructResource, env, term, .{});\n    return res.unpack().payload;\n}\n\"\"\"\n\ntest \"lifecyle operations through terms\" do\n  resource = create_resource_term(47)\n  assert is_reference(resource)\n  assert 47 = retrieve_resource_term(resource)\nend\n```\n\nResources can be marshalled into and out of [`beam.term`](beam.html#term) \nvalues using [`beam.make`](beam.html#make) and [`beam.get`](beam.html#get)\nfunctions as with any other types.\n\n\n\nIn order to convert between the resource type and the wrapped type, you'll\nneed to use the `create` and `unpack` functions.  Note that `create` is\nfailable since under the hood it uses the BEAM resource allocator.\n\n#","ref":"5-resources.html#using-resources-in-functions"},{"type":"extras","title":"Direct marshalling - Resources","doc":"It's possible to directly return resources from a nif function and also\npass them as parameters:  The nif marshalling functions will be able to\ndetect these types and assign them correctly.\n\n```elixir\n~Z\"\"\"\npub fn create_resource_direct(number: u64) !StructResource {\n    return StructResource.create(.{.payload = number}, .{});\n}\n\npub fn retrieve_resource_direct(resource: StructResource) u64 {\n    return resource.unpack().payload;\n}\n\"\"\"\n\ntest \"direct lifecyle operations\" do\n  resource = create_resource_direct(47)\n  assert is_reference(resource)\n  assert 47 = retrieve_resource_direct(resource)\nend\n\ntest \"must be the correct type of reference\" do\n  assert_raise ArgumentError, \"\"\"\n  errors were found at the given arguments:\n\n    * 1st argument: \n  \n       expected: reference (for `beam.Resource(MyStruct, @import(\\\"root\\\"), .{...})`)\n       got: `%{payload: 42}`\n  \"\"\", fn ->\n    retrieve_resource_direct(%{payload: 42})\n  end\n\n  non_resource_ref = make_ref()\n\n  message = \"\"\"\n  errors were found at the given arguments:\n  \n    * 1st argument: \n  \n       expected: reference (for `beam.Resource(MyStruct, @import(\\\"root\\\"), .{...})`)\n       got: `#{inspect non_resource_ref}`\n       note: the reference passed is not associated with a resource of the correct type\n  \"\"\"\n\n  assert_raise ArgumentError, message, fn ->\n    retrieve_resource_direct(non_resource_ref)\n  end\nend\n```\n\n> #","ref":"5-resources.html#direct-marshalling"},{"type":"extras","title":"no coercion {: .warning } - Resources","doc":">\n> You can't pass a term of the same type as the wrapped type and use it within\n> the function.","ref":"5-resources.html#no-coercion-warning"},{"type":"extras","title":"Wrapping pointers and cleanup - Resources","doc":"In many cases you won't want to move large data structures into or out of\nthe resource-allocated memory space; this incurs a data copy cost.  In that\ncase, you might want to store a pointer in the resource memory space.\n\nIn order to do properly clean up after this, you'll need to write a \ncallback function and store it in a struct namespace that gets associated\nwith the resource type in using the `beam.Resource` function `.Callbacks`\noption.\n\nNote that the destructor can also be used in cases where other resources need \nto be cleaned up, for example file descriptors.\n\nThe following functions are supported in the Callbacks, and are all optional.\n\n- `dtor`: called when the GC collects the \n- `stop`: called on stop on behalf of `e.enif_select`\n- `down`: called on resource down, on behalf of `e.enif_monitor_process`\n- `dyncall`: called on dynamic resource call, on behalf of `enif_dynamic_resource_call`\n\n```elixir\n~Z\"\"\"\npub const PointerResource = beam.Resource(*MyStruct, root, .{.Callbacks = PointerResourceCallbacks});\n\npub const PointerResourceCallbacks = struct {\n    pub fn dtor(env: beam.env, s: **MyStruct) void {\n        _ = env;\n        beam.raw_allocator.destroy(s.*);\n    }\n};\n\npub fn create_pointer_resource(number: u64) !PointerResource {\n    const new_struct = try beam.raw_allocator.create(MyStruct);\n    new_struct.payload = number;\n    return PointerResource.create(new_struct, .{});\n}\n\npub fn retrieve_pointer_resource(resource: PointerResource) u64 {\n    return resource.unpack().*.payload;\n}\n\"\"\"\n\ntest \"pointer-based lifecyle operations\" do\n  resource = create_pointer_resource(47)\n  assert is_reference(resource)\n  assert 47 = retrieve_pointer_resource(resource)\nend\n```\n\n> #","ref":"5-resources.html#wrapping-pointers-and-cleanup"},{"type":"extras","title":"pointer allocation strategy {: .warning } - Resources","doc":">\n> It is strongly recommended to use [`beam.raw_allocator`](beam.html#raw_allocator) \n> for your pointer payload allocators, as [`beam.allocator`](beam.html#allocator)\n> is undefined in the callback context.\n>\n> alternatively, if you do use your own managed allocator, you can pack a\n> pointer to the allocator into your datastructure and use this pointer to\n> clean up.\n\n#","ref":"5-resources.html#pointer-allocation-strategy-warning"},{"type":"extras","title":"Release and Keep - Resources","doc":"Resources are associated with `release` and `keep` functionality. \nThese increment and decrement the reference count on the resource which\nallows a nif to prevent the GC from destroying the memory and calling the\ndestructor.   Normally the nif management functions select default \n`release` and `keep` settings so that the functions do the most expected\noutcome - nif functions keep resources while running and then release them \nwhen they're finished.\n\n> #","ref":"5-resources.html#release-and-keep"},{"type":"extras","title":"release on creation {: .info } - Resources","doc":">\n> A Struct resource is normally released on creation.  This can be disabled\n> by passing `.release = false` into the options parameter of \n> `beam.Resource(...).create(...)`\n\n> #","ref":"5-resources.html#release-on-creation-info"},{"type":"extras","title":"keep on get {: .info } - Resources","doc":"> \n> By default, [`beam.get`](beam.html#get) will keep the resource when the\n> internal term retrieved.  This can be disabled by passing `.keep = false`\n> into the options parameter of `beam.get`\n\n> #","ref":"5-resources.html#keep-on-get-info"},{"type":"extras","title":"get without keeping {: .warning } - Resources","doc":">\n> for wrapped datatypes that require a cleanup step (e.g. pointers) it is not \n> recommended to get without keeping, as there could be a race condition where \n> dereferencing the pointer occurs after another nif running in a different OS \n> thread has performed a cleanup against the same resource.\n\n> #","ref":"5-resources.html#get-without-keeping-warning"},{"type":"extras","title":"release when function argument {: .info } - Resources","doc":">\n> If a function is passed a `beam.Resource(...)` type, it will release it\n> at the end of the call.  This can be disabled by setting `:noclean`\n> flag in the function argument options. (see [nif options](4-nif_options.html#noclean))\n\nThese functions are provided in the resource type as \n`beam.Resource(...).release(...)` and `beam.Resource(...).keep(...)`\nfunctions, respectively:\n\n```elixir\n~Z\"\"\"\npub fn release(resource: StructResource) void {\n    resource.release();\n}\n\npub fn keep(resource: StructResource) void {\n    resource.keep();\n}\n\"\"\"\n```","ref":"5-resources.html#release-when-function-argument-info"},{"type":"extras","title":"CXX integration","doc":"# CXX integration\n\nZigler offers several tools to integrate your code with C and C++ code.","ref":"6-c_integration.html"},{"type":"extras","title":"compiling C using the C toolchain - CXX integration","doc":"If you want to compile C or C++ files using the C and C++ toolchain bundled\nwith the zig programming language, you should include a path to your\ninclude directory and a source path or list of source paths to be compiled,\nvia the `include_dir` and `c_src` module options.\n\n> #","ref":"6-c_integration.html#compiling-c-using-the-c-toolchain"},{"type":"extras","title":"source paths {: .info } - CXX integration","doc":">\n> source paths may contain a trailing *, which will pull all c or c++ files\n> in that directory, (but not subdirectory)\n\n#","ref":"6-c_integration.html#source-paths-info"},{"type":"extras","title":"included.h - CXX integration","doc":"```c\n// forwarded function definition\nint plus_one(int);\n```\n\n#","ref":"6-c_integration.html#included-h"},{"type":"extras","title":"src/src.c - CXX integration","doc":"```c\nint plus_one(int value) {\n    return value + 1;\n}\n```\n\n```elixir\ndefmodule CompilingC do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler,\n    include_dir: \"include\", \n    c_src: \"src/*\"\n\n  ~Z\"\"\"\n  const c = @cImport(@cInclude(\"included.h\"));\n\n  pub const plus_one = c.plus_one;\n  \"\"\"\n\n  test \"c plus one\" do\n    assert 48 = plus_one(47)\n  end\nend\n```","ref":"6-c_integration.html#src-src-c"},{"type":"extras","title":"linking against a C abi library - CXX integration","doc":"This example shows you how to link in a system library (which can be `.a`,\n`.so`, `.obj`, or `.dll`).  Zig will resolve the extension based on the\noperating system native rules.  To use the functions in the library, there\nmust also be an associated `.h` file with `extern` functions.\n\nIn this example we'll use the `cblas_dasum` function, which takes a length,\nan pointer to double-precision floating point list, and a integer stride.\nThe result is a sum of the numbers in the list.\n\nThe [rules for collections](#2-collections.html) apply to functions that\nare directly imported from C files.\n\n```elixir\nif {:unix, :linux} == :os.type() do\n# currently we only have access to the BLAS library on linux CI actions, so\n# it's unavailable for other operating systems for automated testing purposes\n\n  defmodule LibraryTest do\n    use ExUnit.Case, async: true\n    use Zig, \n      otp_app: :zigler,\n      link_lib: {:system, \"blas\"}\n\n    ~Z\"\"\"\n    pub const dasum = @cImport(@cInclude(\"cblas.h\")).cblas_dasum;\n    \"\"\"\n\n    test \"dasum\" do\n      assert 6.0 == dasum(3, [1.0, 2.0, 3.0], 1)\n    end\n  end\n```\n\n> #","ref":"6-c_integration.html#linking-against-a-c-abi-library"},{"type":"extras","title":"linking against libcpp: {: .info } - CXX integration","doc":">\n> if you need to link against `libcpp`, the library has a special-cased option: \n> `link_libcpp: true`","ref":"6-c_integration.html#linking-against-libcpp-info"},{"type":"extras","title":"Easy-C - CXX integration","doc":"It's also possible to also automatically create nifs without writing zig \nfunction shims.  This works either with linking an external library with \n`link_lib` or building your own code with `c_src`.\n\nBecause the C ABI exposes all functions publically in a global namespace, \nwe can't use zigler's automatic detection to decide which functions to\nsurface.\n\nHere are the steps to using `easy_c`:\n\n- declare which header file you'd like to use with the `easy_c` module option\n  - you may want to add `include_dir` if the header isn't a system C header.\n- add a `link_lib` or `c_src` option to make sure that the functions are built.\n- declare which functions you'd like to hoist into the module.\n\nIn this example we'll use the `cblas_daxpy` function, which takes a length,\na double-precision `a` value, a list (`x`) of double-precision values, a stride\nfor the `x` values, a list (`y`) of double-precision values, and a stride for \n`y`. \n\nIt then calculates the linear transformation `ax + y` using these vectors.\n\n> ### \"in-out\" parameters {: .info }\n>\n> The `y` parameter in this function is an \"in-out\" parameter.  Instead of \n> returning a result, the results are written into the y pointer.\n>\n> to specify that it's an in-out parameter, we put an integer (which is the\n> 0-indexed index of the argument which is the out parameter).\n>\n> since the returned pointer doesn't have a specified value, we have to \n> specify a `length` option, which could either be a fixed number, or\n> `{:arg, n}` where `n` is the argument index corresponding to a variable\n> length.\n>\n> We can also use [nif options like alias](4-nif_options.html#alias)\n> and [the return option `binary`](2-collections.html#selecting-output-type)\n> alongside these special `easy_c` options.\n\n```elixir\n  defmodule EasyCTest do\n    use ExUnit.Case, async: true\n    use Zig, \n      otp_app: :zigler,\n      easy_c: \"cblas.h\",\n      link_lib: {:system, \"blas\"},\n      nifs: [\n        cblas_daxpy: [return: [4, length: {:arg, 0}]],\n        cblas_daxpy_bin: [alias: :cblas_daxpy, return: [4, :binary, length: {:arg, 0}]]\n      ]\n\n    test \"daxpy as a list\" do\n      assert [7.0, 11.0, 15.0] == cblas_daxpy(3, 3.0, [1.0, 2.0, 3.0], 1, [4.0, 5.0, 6.0], 1)\n    end\n\n    test \"daxpy as a binary\" do\n      assert  > ==\n               cblas_daxpy_bin(3, 3.0, [1.0, 2.0, 3.0], 1, [4.0, 5.0, 6.0], 1)\n    end\n  end\nend\n# module\n```","ref":"6-c_integration.html#easy-c"},{"type":"extras","title":"Nif concurrency strategies","doc":"# Nif concurrency strategies\n\nWhen execution flow enters a Nif, control is fully relinquished from the \nmanaged environment of the BEAM VM to a context where the BEAM is more or\nless unaware of what is going on.\n\nIn general the VM cannot tolerate native code running for [longer than\napproximately one millisecond](https://www.erlang.org/doc/man/erl_nif.html#lengthy_work).\n\nThere are several tools that the BEAM nif system provides for you to","ref":"7-concurrency.html"},{"type":"extras","title":"Synchronous - Nif concurrency strategies","doc":"The default mode for Nifs to run is *synchronous*.  Only use this mode if\nyou are confident that your code can run in under 1ms.","ref":"7-concurrency.html#synchronous"},{"type":"extras","title":"Dirty CPU - Nif concurrency strategies","doc":"`dirty_cpu` mode is usable when your VM has created *Dirty CPU schedulers*.  By\ndefault, the VM creates one dirty CPU scheduler per CPU core available to it.\nNifs tagged as `dirty_cpu` are allowed to run longer than 1 millisecond.\n\nIn order to tag a function as `dirty_cpu`, use the `:dirty_cpu` flag in the\noptions list for the function in the `:nif` call.\n\n#","ref":"7-concurrency.html#dirty-cpu"},{"type":"extras","title":"beam.yield in Dirty CPU - Nif concurrency strategies","doc":"The [`beam.yield`](beam.html#yield) function in dirty CPU mode will detect\nif the parent process has died and will return `error.processterminated`.\n\n```elixir\ndefmodule DirtyCpu do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler,\n    nifs: [long_running: [:dirty_cpu]]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const e = @import(\"erl_nif\");\n  // this is a dirty_cpu nif.\n  pub fn long_running(env: beam.env, pid: beam.pid) !void {\n      // following code triggered when process is killed.\n      defer {\n        // note that the environment of the parent process is dead,\n        // so we have to manually create a new environment and send\n        // from it.\n\n        const env2 = beam.alloc_env();\n        const msg = beam.make(env2, .killed, .{});\n        var pid2 = pid;\n        _ = e.enif_send(null, &pid2, env2, msg.v);\n        beam.free_env(env2);\n      }\n\n      while(true) {\n          _ = try beam.send(env, pid, .unblock);\n          try beam.yield(env);\n      }\n  }\n  \"\"\"\n\n  test \"dirty cpu can be cancelled\" do\n    this = self()\n    dirty_cpu = spawn(fn -> long_running(this) end)\n    assert_receive :unblock\n    Process.exit(dirty_cpu, :kill)\n    assert_receive :killed\n  end\nend\n```\n\n> #","ref":"7-concurrency.html#beam-yield-in-dirty-cpu"},{"type":"extras","title":"queue limitations {: .warning } - Nif concurrency strategies","doc":">\n> if you consume all of your dirty cpu schedulers with nif calls, the next\n> `dirty_cpu` call will block until a scheduler frees up; this could cause\n> undesired latency characteristics.","ref":"7-concurrency.html#queue-limitations-warning"},{"type":"extras","title":"Dirty IO - Nif concurrency strategies","doc":"It's not recommended to use `dirty_io` unless you're performing IO operations\nand blocking using nif events and blocking operations. \n\nIn order to tag a function as `dirty_io`, use the `:dirty_io` flag in the\noptions list for the function in the `:nif` call.","ref":"7-concurrency.html#dirty-io"},{"type":"extras","title":"Threaded - Nif concurrency strategies","doc":"`threaded` mode is usable when your OS supports spawning threads.  This is\neffectively all current platforms supporting the BEAM VM today.  Zigler\nwill wrap your function code \n\nIn order to tag a function as `threaded`, use the `:threaded` flag in the\noptions list for the function in the `:nif` call.  Generally, no other\nchanges must be made to execute a function in threaded mode.\n\n> #","ref":"7-concurrency.html#threaded"},{"type":"extras","title":"env in Threaded mode {: .warning } - Nif concurrency strategies","doc":"> \n> The `env` variable when you run in threaded mode is *not* a process-bound\n> environment.\n\n#","ref":"7-concurrency.html#env-in-threaded-mode-warning"},{"type":"extras","title":"beam.yield in Threaded mode - Nif concurrency strategies","doc":"The [`beam.yield`](beam.html#yield) function in dirty CPU mode will detect\nif the parent process has died and will return `error.processterminated`.\n\n> #","ref":"7-concurrency.html#beam-yield-in-threaded-mode"},{"type":"extras","title":"return from yield quickly! {: .warning } - Nif concurrency strategies","doc":"> \n> You must return from the yield quickly (within 750us).  If you are \n> unable to return quickly, then zigler run will cause the thread \n> metadata to leak.  This will be fixed in zigler 0.11.\n\n```elixir\ndefmodule Threaded do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler,\n    nifs: [long_running: [:threaded]]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const std = @import(\"std\");\n  pub fn long_running(env: beam.env, pid: beam.pid) !void {\n      // following code triggered when process is killed.\n      // note that unlike dirty functions, the lifetime of \n      // env matches the lifetime of the function.\n\n      defer {\n        _ = beam.send(env, pid, .killed) catch {};\n      }\n\n      while(true) {\n          _ = try beam.send(env, pid, .unblock);\n          try beam.yield(env);\n      }\n  }\n  \"\"\"\n\n  @tag :threaded\n  test \"threaded can be cancelled\" do\n    this = self()\n    threaded = spawn(fn -> long_running(this) end)\n    #assert_receive :unblock\n    Process.sleep(100)\n    Process.exit(threaded, :kill)\n    assert_receive :killed\n    Process.sleep(1000)\n  end\nend\n```","ref":"7-concurrency.html#return-from-yield-quickly-warning"},{"type":"extras","title":"Yielding - Nif concurrency strategies","doc":"> #","ref":"7-concurrency.html#yielding"},{"type":"extras","title":"yielding nifs {: .error } - Nif concurrency strategies","doc":">\n> Yielding nifs are not available in this release of Zigler\n\n```elixir\n# module\n```","ref":"7-concurrency.html#yielding-nifs-error"},{"type":"extras","title":"Global module options","doc":"# Global module options","ref":"8-module_options.html"},{"type":"extras","title":"module callbacks - Global module options","doc":"Sometimes it's necessary to execute nif code on module lifecycle events.  Zigler\nsupports binding function into these \n\n> #","ref":"8-module_options.html#module-callbacks"},{"type":"extras","title":"pub and ignore {: .info } - Global module options","doc":"> callback functions must be exported from the module nifs definition code as\n> `pub` functions.\n>\n> Don't forget to include any callback functions in the [`ignore`](4-nif_options.html#ignore)\n> list.\n\n> #","ref":"8-module_options.html#pub-and-ignore-info"},{"type":"extras","title":"callback api {: .info } - Global module options","doc":"> the callback system api may get revised in the future to enable better features\n> and make the priv data system typesafe.\n\n#","ref":"8-module_options.html#callback-api-info"},{"type":"extras","title":"load - Global module options","doc":"The `load` callback is triggered when the module is first loaded.  This\nis useful for, for example, setting up global configuration for the\nmodule, and saving data to be accessed using the [`enif_priv_data`](https://www.erlang.org/doc/man/erl_nif.html#enif_priv_data) function.\n\n> #","ref":"8-module_options.html#load"},{"type":"extras","title":"load function type signature {: .info } - Global module options","doc":">\n> The type signature of the load function is expected to be:\n> \n> `fn (beam.env, [*c]?*anyopaque, beam.term)`\n>\n> The second term is a pointer to the location of the priv data.  the load \n> function can optionally set this pointer, which will enable access to\n> that priv_data pointer via `enif_priv_data`.\n>\n> This API may change in the future to avoid the opaque c double pointer.\n\n```elixir\ndefmodule ZiglerTest.LoadTest do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler, \n    callbacks: [\n      on_load: :load_function,\n    ],\n    ignore: [:load_function]\n\n  ~Z\"\"\"\n  const beam = @import(\"beam\");\n  const e = @import(\"erl_nif\");\n  const std = @import(\"std\");\n\n  var priv_data: u64 = undefined; \n\n  pub fn load_function(env: beam.env, priv_data_ptr: [*c]?*anyopaque, init_term: beam.term) c_int {\n      priv_data = 47;\n      priv_data_ptr.* = &priv_data;\n      _ = init_term;\n      _ = env;\n      return 0;\n  }\n\n  pub fn get_priv_data(env: beam.env) u64 {\n      const priv_data_ptr: *u64 = @ptrCast(@alignCast(e.enif_priv_data(env)));\n      return priv_data_ptr.*;\n  }\n\n  pub fn loaded_value() u64 {\n      return priv_data;\n  }\n  \"\"\"\n\n  test \"module load\" do\n    assert 47 = loaded_value()\n    assert 47 = get_priv_data()\n  end\n\nend\n```\n\n> #","ref":"8-module_options.html#load-function-type-signature-info"},{"type":"extras","title":"load_nif data {: .error } - Global module options","doc":">\n> The third term (`init_data`) is supplied on invocation of `:erlang.load_nif/2`.  \n> Assigning this value is not supported in this version of zigler.\n\n#","ref":"8-module_options.html#load_nif-data-error"},{"type":"extras","title":"upgrade - Global module options","doc":"The `upgrade` callback is triggered when the module is upgraded from a\nprevious version of the module during a hot swap event.\n\n> #","ref":"8-module_options.html#upgrade"},{"type":"extras","title":"upgrade not supported {: .error } - Global module options","doc":">\n> the upgrade callback is not supported in this version of zigler\n\n#","ref":"8-module_options.html#upgrade-not-supported-error"},{"type":"extras","title":"unload - Global module options","doc":"The `unload` callback is triggered when the module is deregistered from\nthe BEAM module catalog.\n\n> #","ref":"8-module_options.html#unload"},{"type":"extras","title":"unload not supported {: .error } - Global module options","doc":">\n> the unload callback is not supported in this version of zigler","ref":"8-module_options.html#unload-not-supported-error"},{"type":"extras","title":"adding packages - Global module options","doc":"It's possible to add zig files as packages using the `packages` \nkeyword option.  The name of the package is the key, and the value \nis a tuple of the path to the zig file that acts as the package and\na list of dependencies for the package.  \n\n#","ref":"8-module_options.html#adding-packages"},{"type":"extras","title":"Example extra.zig - Global module options","doc":"```zig\npub const value = 47;\n```\n\n```elixir\ndefmodule PackageFile do\n  use ExUnit.Case, async: true\n  use Zig, \n    otp_app: :zigler,\n    packages: [extra: {\"test/_support/package/extra.zig\", [:beam]}]\n\n  ~Z\"\"\"\n  const extra = @import(\"extra\");\n\n  pub fn extra_value() u64 {\n    return extra.value;\n  }\n  \"\"\"\n\n  test \"package file\" do\n    assert 47 = extra_value()\n  end\nend\n#module\n```","ref":"8-module_options.html#example-extra-zig"}],"content_type":"text/markdown"}